//
//		Sample C++ application using the Borland IOSTREAM library and complex
//		numbers to calcuate the voltage of an AC circuit using the equations
//
//			Z = R + jwL + 1/(jwC)
//			V = ZI
//
//		Adapted from the book "Programing in C++" by Dewhurst and Stark.
//
//		This example puts everything together, C++, stream I/O, floating point,
//		and dynamic memory management.
//

#pragma	warn	-lvc						// Ignore temporaries

#include	<iostream.h>					// Access to stream I/O
#include	<complex.h>						// Access to complex numbers


void		main()
{
	const	complex	j(0, 1) ;			// imaginary 1
	const	double	pi = 3.14159 ;		// Approximation to pi

	double	L = .03,						// Inductance, in henries
				R = 5000,					// Resistance, in ohms
				C,								// Capacitance, in farads
				freq = 60,					// Frequency in hertz
				omega = 2 * pi * freq ;	//	Frequency in radians

	complex	I(0, 0),						// Current
				Z,								// Impedance
				*V ;							// Voltage


	// Use new to allocate our voltage object
	V = new complex() ;
	
	// Run forever, ramping the current and changing the capacitance
	for (C = .02;; I += .00001, C += .0001)   {
		// Calculate the impedance and the voltage
		Z = R + j * omega * L + 1 / (j * omega * C) ;
		*V = Z * I ;

		// Display the voltage on the console
		cout << *V << endl ;
	}
}
