/*
//		Sample application to demonstrate the use of the Borland C++ math
//		coprocessor emulation in an embedded application.  If an actual
//		math coprocessor is available, the MAKEFILE macro FLOAT needs to
//		be changed.
*/

#include	<stdlib.h>
#include	<signal.h>
#include	<setjmp.h>
#include	<math.h>
#include	<float.h>

#define	ERR_LIMIT		0.000001

void		MyFPHandler(int, int) ;
jmp_buf	jbuf ;


/*
//		This function gains control of floating point exceptions
*/

void		MyFPHandler(
				int		sig,				/* SIGFPE */
				int		type				/* Floating point subtype */
			)
{
	/* Reset the math coprocessor */
	_fpreset() ;

	/* Restore the intercept for this signal */
	signal(sig, MyFPHandler) ;
	longjmp(jbuf, type) ;
}


void		main(void)
{
	float		f1, f2 ;
	double	d1, d2 ;
	unsigned	errors ;


	/* Run this demo forever */
	for (;;)   {
		/* Initialize the error count and set the default handler */
		errors = 0 ;
		signal(SIGFPE, SIG_DFL) ;

		/* Check that sin(x)^2 + cos(x)^2 = 1 where x is PI/2 */
		f1 = M_PI_2 ;
		f2 = pow(sin(f1), 2.0) + pow(cos(f1), 2.0) ;
		if (fabs(f2 - 1.0) > ERR_LIMIT)
			errors++ ;

		/* Check that ln(e^2) = 2 */
		d1 = M_E ;
		d2 = log(pow(d1, 2.0)) ;
		if (fabs(d2 - 2.0) > ERR_LIMIT)
			errors++ ;

		/* Force an error and see that matherr() catches and fixes it */
		d1 = -4.0 ;
		d2 = sqrt(d1) ;

		/* Install a new floating point exception handler */
		signal(SIGFPE, MyFPHandler) ;

		/* Force an intrinsic error and verify it is intercepted */
		if (setjmp(jbuf) == 0)
			raise(SIGFPE) ;
		else
			errors++ ;							/* This code should also execute */

		/* Force an intrinsic error and verify it is intercepted */
		if (setjmp(jbuf) == 0)   {
			d1 = 0.0 ;
			d2 = 1.0 ;
			d2 = d2 / d1 ;
		}
		else   {
			d2 = M_PI ;							/* This code should also execute */
			errors++ ;
		}

		/* Verify that the correct number of errors were detected */
		f1 = (errors != 2) ? -1.0 : 1.0 ;
	}
}
