/*
//		Sample application to test dynamic memory allocation by randomly
//		allocated and freeing random sized blocks of memory.  This demo
//		requires the Paradigm MS-DOS Int 21h emulation to be installed if
//		the compact or large model is used.
//
//		Memory management takes place on the heap for small data memory
//		models and on the far heap for large data models for Borland C++.
*/

#include	<stdlib.h>						/* malloc(), free() prototypes */
#include	<string.h>						/* memset() prototype */
#include	<alloc.h>						/* heapcheck() prototype */

#define	ALLOCS		250				/* Number of allocations */
#define	BLOCK_SIZE	100				/* Maximum block size */

char	*p[ALLOCS] ;						/* Allocated memory block pointers */


void		main(void)
{
	unsigned	int	item ;				/* Index to block handle */
	unsigned	int	size ;				/* Requested block size */
	unsigned	int	allocs ;				/* Number of successful allocations */
	unsigned int	errors ;				/* Number of failed allocations */
	unsigned int	heapcorrupt ;		/* 1 if the heap is corrupted */


	for (heapcorrupt = allocs = errors = 0;;)   {
		/* Compute the pointer and allocation size */
		item = random(ALLOCS) ;
		size = random(BLOCK_SIZE) + 1 ;

		/* Free up the memory if in use */
		if (p[item] != NULL)
			free(p[item]) ;

		/* Allocate a new block */
		if ((p[item] = malloc(size)) == NULL)
			errors++ ;
		else   {
			allocs++ ;
			memset(p[item], item, size) ;
		}

		/* Check if the heap is intact */
		if (heapcheck() == _HEAPCORRUPT)
			heapcorrupt |= 1 ;
	}
}
