/*
//		Extensible MS-DOS Int 21h Emulation for Embedded Systems
//		Copyright (C) 1991, 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the definitions common to modules implementing
//		the MS-DOS Int 21h emulation for Borland C++.
*/

#if	!defined(_DOSEMU)
#define	_DOSEMU


/*
//		Some commonly emulated MS-DOS functions.  These may or may be
//		implemented, depending on the compiler.
*/

#define	DOS_TERMINATE			0x00
#define	DOS_DIRECTIO			0x06
#define	DOS_GETSTATUS			0x0b
#define	DOS_SETVECTOR			0x25
#define	DOS_VERSION				0x30
#define	DOS_GETVECTOR			0x35
#define	DOS_READ					0x3f
#define	DOS_WRITE				0x40
#define	DOS_IOCTL				0x44
#define	DOS_ALLOC				0x48
#define	DOS_FREE					0x49
#define	DOS_REALLOC				0x4a
#define	DOS_EXIT					0x4c


/*
//		Define the format of the DOS memory control block (MCB).  Only the
//		id and size fields need be implemented since the all memory is assumed
//		to be used by a single application.  If a multitasking system is built,
//		placing the task id in the owner field allows freeing the memory owned
//		by the task when it exits.  An id field of 0 is used to mark an unused
//		block.
//
//		The layout in memory consists of an MCB followed by the memory block.
//		The allocator returns to the caller the segment of the block which
//		can be decremented to find the corresponding MCB.
*/

typedef	struct   {
		UCHAR		id ;						/* 'M' or 'Z' */
		UINT		owner ;					/* Application PID or PSP */
		UINT		size ;					/* Size in paragraphs */
	} MCB, far *MCBP ;					/* Structure and pointer definition */


/*
//		Define the TDREM exit codes
*/

typedef	enum   {
		TDE_DMMERR = 243,					/* Heap initialization error */
		TDE_FPEXCP = 244,					/* Floating point exception */
		TDE_STKOVL = 245,					/* Stack overflow */
		TDE_ABORT = 246,					/* abort() called */
		TDE_INTO = 247,					/* INTO exception */
		TDE_CHKIND = 248,					/* CHKIND exception */
		TDE_IOTRAP = 249,					/* I/O trap exception */
		TDE_ILLOP = 250,					/* Illegal opcode exception */
		TDE_ESCOP = 251,					/* Escape opcode exception */
		TDE_ZERODIV = 252					/* Zero divide exception */
	} TDEXIT ;								/* TDREM exit codes */


/*
//		Function prototypes for the console I/O interface.
*/

extern	void		InitConsole(void) ;
extern	int		_getch(void) ;
extern	int		_putch(int) ;

extern	UINT		wInt21Hanlders ;

#endif	/* !defined(_DOSEMU) */
