/*
//		Paradigm LOCATE _fperr() Replacement for Borland C++ 3.10
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This routine is a replacement for the Borland C++ 3.10 run-time
//		library _fperr() function.  This is the routine is called when
//		an intrinsic floating point exception occurs and defines the
//		action to be taken in response to the exception.
//
//		The default run-time library routine is not suitable for use in an
//		embedded application.  The function _fperr() is called from the
//		floating point exception handler defined in the file BCPPFLT.ASM.
//
//		NOTE:	This function is application dependent and must be modified
//				for use in the target system.
*/

#include	<stddef.h>						/* Definition of NULL */
#include	<signal.h>						/* Signal handling mechanisms */

#include	<math.h>							/* Access _mexcep */
#include	<float.h>						/* Access to the FPE types */

#define	FPE_DENORMAL		130		/* 80x87 denormalized operand	*/


/*
//		The only exceptions routed through this code are GENUINE floating
//		point ones.  The explicitly raised SIGFPE and the integer related
//		SIGFPE types DO NOT come this way.
*/
typedef	struct   {
				int	FPEtype ;			/* Exception type */
				char	*string ;			/* ASCIIZ name of the execption */
			} FLT_ERR ;

/*
//		This is the definition of the floating point exception structure.
//		It describes the info passed to _fperr(), the purpose of which is
//		to give a C interface to the trapping of floating point exceptions
//		in a style similar to matherr().
//		
//		_fperr() is called directly from the floating point exception handler.
//		Portable floating point exception handlers should call signal() instead,
//		as that is an ANSI function.  _fperr() is for lower level control over
//		the exception and is provided to give the user control over the default
//		exception hanlding and eliminate the Borland C++ DOS dependancies.
//		
//		The default _fperr() only uses the type and subtype fields of the
//		structure fpexcep.  The other fields are correctly filled in if the
//		coprocessor is present, but not otherwise.  They can be used to
//		help track down the source of the error, the circumstances causing
//		the exception and the potential corrective action.
//		
//		_fperr() is not called for denormal exceptions, as these are handled by
//		the FP exception handler.  The FP exception handler also attempts to
//		correct some invalid operation exceptions, and only calls _fperr() if it
//		is unsuccessful in curing the problem.
//		
//		Note that the subtype field is always 1.  Future implementations may use
//		this field to pass more info, such as whether an INVALID exception
//		came from a FSQRT instruction.
//		
//		Here are the reasonable alternatives for a custom floating point
//		exception handler.
//		
//		1.	Print a suitable message and exit.  This is what the default
//			Borland C++ handler does and is not suitable for embedded systems.  A
//			program that wants to do the same may still wish to replace the
//			handler as it may have some additional cleaning up to do, or may want
//			to print a more informative message.  Probably not an option for an
//			embedded application.
//		
//		2. Do a long jump to safe place in the program.  If so, the
//			program must pay attention to all of the usual hazards of long
//			jumps, and in addition, it should call _fpreset() to reset the
//			coprocessor or emulator.  Since interrupts occur asynchronously,
//			there is more danger than usual that the code will be left in an
//			inconsistent state.  This is preferred since the coprocessor or
//			emulator state may be in unsuitable to continue from the point of
//			the exception.
//		
//		3. Ignore the exception.  In most cases the coprocessor will
//			generate infinities and NANs which are likely to cause additional
//			exceptions.  These exceptions can be ignored more efficiently by
//			using _control87() to mask them.  This option does not work if
//			the coprocessor is being emulated, as the emulator does not support
//			all of the exception handling that the 8087 does.
//		
//		4. Set a flag and continue.  As with case 3 above, most programs
//			may prefer the simpler strategy of masking the exceptions.  The
//			occurrence of the exception can still be detected by examining
//			the status word with _status87() and can be cleared with
//			_clear87().  This option does not work if the coprocessor is being
//			emulated.
//		
//		5. Attempt to analyze the damage and repair it. This is nearly
//			impossible as the 80x87 is a very complex chip with many
//			instructions, data types, registers, and special cases.  Some
//			info is provided at the _fperr() level for programs to try.
//
//		Normally floating point exceptions should not occur in a well-designed
//		and debugged application.  Since handling exceptions should they occur
//		is extremely application-dependent, further actions are beyond the
//		scope of this function.
*/

typedef	struct   {
		_mexcep			type ;			/* Exception type */
		int				subtype ;		/* Exception subtype */
		unsigned int	opcode ;			/* Offending instruction */
		void	huge		*datap ;			/* Pointer to the operand, if any */
		void	(huge *codep)() ;			/* Pointer to the instruction */
	} FPexcp, _ss *FPexcpPtr ;

typedef	void	(* cdecl CatcherPTR)() ;

extern	CatcherPTR (* cdecl __SignalPtr)();

static	FLT_ERR	_fpe_msg[] = {
				{	FPE_INVALID,		"Domain"									},
				{	FPE_DENORMAL,		"Denormal"								},
				{	FPE_ZERODIVIDE,	"Divide by 0"							},
				{	FPE_OVERFLOW,		"Overflow"								},
				{	FPE_UNDERFLOW,		"Underflow"								},
				{	FPE_INEXACT,		"Partial loss of precision"		},
				{	FPE_STACKFAULT,	"Stack fault"							}
			} ;


void	cdecl	_fperror(void)
{
	FPexcpPtr	a = (FPexcpPtr) _BX ;


	/* Check if an SIGFPE handler has been installed */
	if (__SignalPtr != NULL)   {
		CatcherPTR	ptr ;

#pragma warn -pro
		/* Get the current signal setting */
		ptr = (*__SignalPtr)(SIGFPE, SIG_DFL) ;

		/* Reset the signal to the original value */
		(*__SignalPtr)(SIGFPE, ptr) ;

		/* Test if to ignore the signal or service it */
		if (ptr != (void (*)()) SIG_IGN)   {
			/* Test what action to take */
			if (ptr == (void (*)()) SIG_DFL)
				goto l1 ;

			(*__SignalPtr)(SIGFPE, SIG_DFL) ;
			(*ptr)(SIGFPE, _fpe_msg[a->type-1].FPEtype) ;
		}
	}
#pragma warn .pro
	else   {
l1:
		/* Reset the 80x87/emulator */
		_fpreset() ;

		/* User-customized application code belongs here */
		;
	}
}
