/*
//		Paradigm LOCATE _matherr() Replacement for Borland C++ 3.10
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		The _matherr() function is the interface between the math run-time
//		library and the user-customized matherr() function.  It is the
//		responsibility of matherr() to handle all exceptions since except
//		for setting errno, _matherr() has no other impact and will return to
//		the caller with the result to be used.
*/

#include	<stdlib.h>						/* NULL and errno definitions */
#include	<math.h>							/* Floating point definitions */


/*
//		Function:	_matherr
//
//		_matherr() serves as a focal point for error handling in all
//		math library functions; it calls matherr() and processes the
//		return value from matherr().  _matherr() should never be called
//		directly by user programs.
//
//		Whenever an error occurs in one of the math library routines _matherr()
//		is called with several arguments.  _matherr() does four things:
//			- It uses its arguments to fill out an exception structure.
//			- It calls matherr() with e, a pointer to the exception structure,
//			  to  see if matherr() can resolve the error.
//			- It examines the return value from matherr() as follows:
//					o If matherr returns 0, (indicating that it was not able to
//					  resolve the error) _matherr() sets errno and returns.
//					o If matherr returns non-zero, (indicating that it was able
//					  to resolve the error) _matherr() is silent; it does not set
//					  errno.
//					o It returns e->retval to the original caller.  Note
//					  that matherr() might modify e->retval to specify the value
//					  it wants propagated back to the original caller.
//
//			- When _matherr() sets errno (based on a 0 return from matherr()),
//			  it maps the kind of error that occurred (from the type field in
//			  the exception structure) onto an errno value of either EDOM or
//			  ERANGE.
//
//		Return value:
//			_matherr() returns the value, e->retval to the caller.
*/

double	cdecl	_matherr(
				_mexcep	why,				/* Reason for the exception */
				char		*fun,				/* Name of the RTL function */
				double	*arg1p,			/* First argument */
				double	*arg2p,			/* Second argument */
				double	retval			/* Default return value */
			)
{
	struct	exception	e ;			/* Exception structure for matherr() */


	/* Initialize the exception structure for the call to matherr() */
	e.type = why ;
	e.name = fun ;
	e.arg1 = (NULL == arg1p) ? 0 : *arg1p ;
	e.arg2 = (NULL == arg2p) ? 0 : *arg2p ;
	e.retval = retval ;

	/* Call matherr() and abort if it can't handle the exception */
	if (matherr(&e) == 0)   {
		switch (why)   {
			case DOMAIN:
			case SING:
				errno = EDOM ;
				break ;
				
			case OVERFLOW:
			case UNDERFLOW:
			case TLOSS:
			case PLOSS:
				errno = ERANGE ;
				break ;
		}
	}

	/* Return the value to be used */
	return	e.retval ;
}
