/*
//		Microsoft C 5.10 MS-DOS Int 21h Emulation, Version 2.00
//		Copyright (C) 1989 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the code for serving common MS-DOS requests from
//		the Microsoft C 5.10 run-time library.
*/

#include	<dos.h>							/* I/O port manipulations */

#include	"typedefs.h"					/* General purpose type declarations */
#include	"dosemu.h"						/* Int 21h function/data prototypes */


/*
//		Function:	Int21Handler	
//
//		This interrupt handler services any Int 21h calls from the application.
//		Since every application will have different requirements, the scope of this
//		handler will vary.  As implemented here, sufficient functionality exists
//		for supporting floating point arithmetic and console I/O.
*/

void far	interrupt Int21Handler(
				WORD		es,				/* Registers as saved on the stack */
				WORD		ds,
				WORD		di,
				WORD		si,
				WORD		bp,
				WORD		sp,
				WORD		bx,
				WORD		dx,
				WORD		cx,
				WORD		ax,
				WORD		ip,
				WORD		cs,
				WORD		psw
		)
{
	WORD		i ;							/* General purpose counter */
	WORD		j ;							/* General purpose counter */
	BYTE_PTR	pByte ;						/* General purpose BYTE pointer */
	BYTE_PTR	cp ;							/* General purpose BYTE pointer */
	WORD_PTR	wp ;							/* General purpose WORD pointer */
	int		c ;							/* Temporary character storage */


	/* Permit interrupts while in the Int 21h emulation */
	_enable() ;

	/* Clear the carry to indicate success */
	psw &= 0xfffe ;

	/* The contents of the AH register is used to determine the service */
	switch (highbyte(ax))   {
		case DOS_CINPUT:
			ax = _getch() ;
			break ;

		case DOS_DIRECTIO:
			if (lowbyte(dx) == 0xff)
				c = _getch() ;
			else
				_putch(lowbyte(dx)) ;
			break ;

		case DOS_BUFINPUT:
			cp = (BYTE_PTR) MK_FP(ds, dx) ;
			j = *cp++ ;
			pByte = cp++ ;
			for (i = 0; i < j; )   {
				c = _getch() ;
				/* Check for and process backspace characters */
				if (c == '\b' && i > 0)   {
					_putch(c) ;
					_putch(' ') ;
					_putch(c) ;
					cp-- ;
					i-- ;
					continue ;
				}
				/* Check for the end of the line */
				if (c == '\r')   {
					_putch(*cp++ = '\r') ;
					_putch('\n') ;
					*pByte = i ;
					break ;
				}
				/* Put the character in the buffer and echo */
				_putch(*cp++ = c) ;
				i++ ;
			}
			break ;

		case DOS_SETVECTOR:
			wp = (WORD_PTR) MK_FP(0, lowbyte(ax) << 2) ;
			_disable() ;
			*wp++ = dx ;
			*wp = ds ;
			_enable() ;
			break ;
		
		case DOS_VERSION:
			ax = 0x0a02 ;					/* Simulate MS-DOS 2.10 */
			break ;

		case DOS_GETVECTOR:
			wp = (WORD_PTR) MK_FP(0, lowbyte(ax) << 2) ;
			_disable() ;
			bx = *wp++ ;
			es = *wp ;
			_enable() ;
			break ;

		default:		 						/* All other functions */
			psw |= 1 ;						/* Set the carry flag */
			break ;
	}

	/* Return with the carry flag indicating the state of the result */
	return ;
}


/*
//		Function:	Int07Handler	
//
//		This handler needs to simply add the size of the ESC instruction
//		to the IP on the stack and return, effectively skipping the
//		instruction.  This is a necessary precaution since depending on the
//		processor used, some 8087 instructions may be executed to test for
//		the presence of a coprocessor (typically a store status) and cause
//		an exception.
//
//		This support is needed by the 80C186/188 and NEC V25/V35
//		microprocessors.
*/

void	interrupt far	Int07Handler(
				WORD		es,				/* Registers as saved on the stack */
				WORD		ds,
				WORD		di,
				WORD		si,
				WORD		bp,
				WORD		sp,
				WORD		bx,
				WORD		dx,
				WORD		cx,
				WORD		ax,
				WORD		ip,
				WORD		cs,
				WORD		psw
		)
{
	BYTE_PTR	pOpcode ;					/* Opcode address */

	/* Make a pointer to the opcode */
	pOpcode = (BYTE_PTR) MK_FP(cs, ip + 1) ;

	switch (*pOpcode & 0xc0)   {
		case 0x00:
			ip += ((*pOpcode & 7) == 6) ? 4 : 2 ;
			break ;
		case 0x40:
			ip += 3 ;
			break ;
		case 0x80:
			ip += 4 ;
			break ;
		case 0xc0:
			ip += 2 ;
			break ;
	}
	return ;
}


/*
//		Function:	InitInt21h
//
//		This function is responsible for initializing the Paradigm Systems
//		MS-DOS Int 21h emulation package.
//
//		Specific initializations include:
//				DOS emulation interrupt handlers
//				Serial console
//				Other miscellaneous initializations
*/

void	far	InitInt21h(void)
{
	/* Install the interrupt handlers */
	setvect(0x21, Int21Handler) ;
	setvect(0x07, Int07Handler) ;

	/* Initialize the console hardware */
	InitConsole() ;

	/* Put any other initializations desired here */
	return ;
}
