	page	60, 132
	name	emoem
	title	Microsoft C 5.1 80x87 Exception Handling, Version 2.00

;
;	Embedded System Startup Code for Microsoft C 5.1
;	Copyright (C) 1989 Paradigm Systems.  All rights reserved.
;
;	User customization routines for 8087/80287 coprocessor and emulator
;	floating point packages.  The following routines need to be written
;	to properly handle the 80x87 installation, termination, and interrupt
;	exception handling.
;
;	    __FPINSTALL87	Install 8087 interrupt handler
;	    __FPTERMINATE87 	Deinstall 8087 interrupt handler
;	    __fpintreset	Reset 80x87 interrupt hardware
;
;	Most hardware handles the 8087/80287 in one of the following ways
;	    1.  NMI - IBM PC and clones all handle the interrupt this way
;	    2.  Master 8259A interrupt controller
;	    3.  Slave 8259A interrupt controller
;
;	The Intel 80C186 family reserves the use of Int 10h for math
;	coprocessor exceptions.
;

	INCLUDE	startup.inc		; Macros and other assembly langauge defintions
	INCLUDE	msc51.inc		; MSC 5.1 specific defintions

IF	FLOAT NE FPALT			; Skip if alternate math library

FPVECTOR	= 2			; 80x87 interrupt vector number
FLTINT		= 80h			; Bit in status word for interrupt pending

	subttl	Segment ordering/alignment section
	page
;
;	Segments declared/referenced from this module
;
DefSeg	_data,    word,  public,  DATA,        DGROUP	; Initialized data
DefSeg	emulator_text, word, public, CODE,     <>
DefSeg	emulator_data, word, public, FAR_DATA, <>

emulator_data	segment
LocalW	statwd				; Temporary for status word
LocalW	oldvec				; Original 8087 exception interrupt vector
emulator_data	ends

	subttl	Initialization code section
	page
ExtProc	_FPEXCEPTION87, near		; MSC exception handler

emulator_text	segment
	assume	cs:emulator_text

;
;	Function: _FPINSTALL87
;
;	Perform the initialization of the 80x87 hardware before the application
;	is started.
;

BegProc	_FPINSTALL87, near
	assume	cs:emulator_text, ds:emulator_data
;
;	Save the original interrupt vector to permit chaining in the event the
;	interrupt occurs and it is not caused by the coprocessor.
;
	mov	al, FPVECTOR
	mov	ah, 35h
	int	21h
	mov	wptr [oldvec], bx
	mov	wptr [oldvec+2], es

;
;	Install the exception handling interrupt vector
;
	push	ds
	mov	ah, 25h
	mov	al, FPVECTOR
	mov	dx, offset __fpinterrupt87
	push	cs
	pop	ds
	int	21h
	pop	ds

;
;	Set up any additional interrupt hardware (such an 8259A interrupt
;	controller here.
;
	ret

EndProc	_FPINSTALL87


;
;	Function: _FPTERMINATE87
;
;	Any cleanup when the terminate option has been requested.
;	Currently not implemented for embedded systems.
;
BegProc	_FPTERMINATE87
	assume	cs:emulator_text, ds:emulator_data
	ret
EndProc	_FPTERMINATE87


;
;	Function:  _fpintreset
;
;	This function should reset any hardware associated with a floating
;	point exception to permit the continued operation of the 80x87.
;
;	*** APPLICATION NOTE ***
;	After an floating point exception, the 8087 stack may require fixup.
;	It is the responsibility of the application to ensure that the 8087
;	(real or emulated) is left in a consistent state.
;	*** APPLICATION NOTE ***
;

BegProc	_fpintreset
	assume	cs:emulator_text, ds:emulator_data
	ret				; User customized reset code
EndProc	_fpintreset


;
;	Function:	_fpinterrupt87
;
;	This is the 8087 exception interrupt routine.  All OEM specific
;	interrupt and harware handling should be done in _fpintreset because
;	_FPEXCEPTION87 (the OEM independent 8087 exception handler) may not
;	return.  _FPEXCEPTION87 also turns interrupts back on.
;

	assume	cs:emulator_text, ds:nothing
BegProc	_fpinterrupt87
	push	ds			; Save the modified registers
	push	ax
	mov	ax, emulator_data
	mov	ds, ax
	assume	ds:emulator_data

	fnstsw	[statwd]		; Store out exceptions
IF	FLOAT EQ FP87			; Wait till complete if an 8087
	wait
ENDIF
	test	bptr [statwd], FLTINT	; Test for 8087 interrupt
	pop	ax
	pop	ds
	jz	not87int		; Not an 8087 interrupt - chain

	call	_fpintreset		; OEM interrupt reset routine
	call	_FPEXCEPTION87	 	; 8087 error handling - may not return
					; this routine turns interrupts back on
	iret

not87int:
	push	ds			; Space for the chain address
	push	ax
	mov	ax, emulator_data
	mov	ds, ax
	assume	ds:emulator_data

	push	ax			; Saved registers
	push	bp
	mov	bp, sp

	mov	ax, wptr [oldvec]	; Get the offset
	mov	[bp+4], ax		; Put on the stack

	mov	ax, wptr [oldvec+2]	; Get the segment
	xchg	ax, [bp+6]
	mov	ds, ax			; Restore DS

	pop	bp			; Restore the other registers
	pop	ax
	retf				; Chain to the old handler

EndProc	_fpinterrupt87

emulator_text	ends

ENDIF	; FPALT
	end
