	page	60, 132
	name	msc51
	title	Microsoft C 5.1 Startup Code, Version 2.00

;
;	Embedded System Startup Code for Microsoft C 5.1
;	Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
;
;	Make sure that this startup module is specified first when linking
;	the application.  Requires MASM 5.10 or TASM 1.0 to assemble.
;
;		masm msc51 /mx /d__MDL__ [ /dTDREM ] ;
;		tasm msc51 /mx /d__MDL__ [ /dTDREM ]
;
;	where
;		__MDL__ is one of the following:
;
;		__S__	Small memory model
;		__C__	Compact memory model
;		__M__	Medium memory model
;		__L__	Large memory model
;		__H__	Huge memory model
;
;	If no memory model is specified, the small memory model is
;	assumed and a warning is issued.
;
;	This module is complete in that it supports all memory models for
;	either ROMable or TDREM applications.  Being a "worst case"
;	construction, this startup can be shortened considerably if the
;	application does not make use of the FAR_BSS or FAR_DATA classes.
;
	
	INCLUDE	startup.inc		; Macros and other assembly langauge defintions
	INCLUDE	msc51.inc		; MSC 5.1 specific defintions

	SHOW	<Paradigm LOCATE Microsoft C 5.1 Startup Support>
IFDEF	TDREM
	SHOW	<Building Paradigm TDREM-compatible startup code>
ENDIF	; TDREM

	subttl	Segment ordering/alignment section
	page
;
;	Segment and group declarations.  The order of these declarations is
;	used to control the order of segments in the .EXE file since the
;	Microsoft and Borland linkers copy segments in the same order they
;	are encountered in the object files.
;
;	Segment naming conventions follow Microsoft whenever possible.  Newly
;	defined segments use the following rules
;
;				optional	optional
;				========	========
;		b - begin	f - far		r - ROM		d - DATA
;		e - end						b - BSS
;

DefSeg	btext,    para,  public,  CODE,	       <>	; Start of CODE class
DefSeg	_text,    word,  public,  CODE,	       <>	; Default code
DefSeg	begdata,  para,  public,  DATA,        DGROUP	; Start of DGROUP
DefSeg	_data,    word,  public,  DATA,        DGROUP	; Initialized data
DefSeg	xib,      word,  public,  DATA,        DGROUP
DefSeg	xi,       word,  public,  DATA,        DGROUP	; Near initializers
DefSeg	xie,      word,  public,  DATA,        DGROUP
DefSeg	xifb,     word,  public,  DATA,        DGROUP
DefSeg	xif,      word,  public,  DATA,        DGROUP	; Far initializers
DefSeg	xife,     word,  public,  DATA,        DGROUP
DefSeg	const,    word,  public,  CONST,       DGROUP	; Constant data
DefSeg	hdr,      word,  public,  MSG,         DGROUP
DefSeg	_bss,     word,  public,  BSS,         DGROUP	; Uninitialized data
DefSeg	stack,    para,  stack,   STACK,       DGROUP	; Program stack

;
;	These segments are only used when the application contains large
;	initialized or uninitialized data structures or uses huge data.
;
DefSeg	far_data, para,  public,  FAR_DATA,    <>	; Far initialized data
DefSeg	efd,	  para,  public,  ENDFAR_DATA, <>	
DefSeg	far_bss,  para,  public,  FAR_BSS,     <>	; Far uninitialized data
DefSeg	efb,	  para,  public,  ENDFAR_BSS,  <>

;
;	The following segments are used to mark the place for ROM copies
;	of initialized data for use by the startup code.
;
DefSeg	brd,      para,  public,  ROMDATA,     <>	; Copy of initialized data
DefSeg	erd,      para,  public,  ENDROMDATA,  <>
DefSeg	brfd,     para,  public,  ROMFARDATA,  <>	; Copy of far initialized data
DefSeg	erfd,     para,  public,  ENDROMFARDATA,<>

;
;	Define functions/procedures referenced here but defined elsewhere
;
ExtProc	main
IF	@CodeSize EQ MM_NEAR
ExtProc	_aNahdiff, near
ELSE
ExtProc	_aFahdiff, far
ENDIF	; MM_NEAR

		
public	__acrtused
	__acrtused = 9876h	; Compiled object modules automatically reference
				; this symbol to pull in the startup code from a
				; library

	subttl	Startup/initialization code
	page
_text	segment
	assume	cs:_text

BegProc	_startup, far
IFDEF	TDREM
;	Check if the segment registers have been setup by DOS.  TDREM always
;	clears the DS segment register so it can be used to check if the
;	application is accidently running under MS-DOS.

	mov	ax, es			; See if DOS setup ES
	or	ax, ax			; Check if the PSP is zero
	jz	embedded		; Continue with the embedded code

	push	cs			; Display a DOS error message
	pop	ds
	assume	ds:_text
	mov	dx, offset errmsg
	mov	ah, 09h
	int	21h
	mov	ah, 04ch		; Exit before doing something harmful
	int	21h			; No return from this DOS call
errmsg	db	"Embedded application: DOS not supported", 0ah, 0dh, '$'
embedded:
ENDIF	; TDREM				; MS-DOS error check

	cli				; Disable interrupts
	cld				; Make sure the direction is forward

;
;	Initialize the stack segment and pointer registers
;
	mov	ax, dgroup		; Get the value of the stack segment
	mov	ss, ax			; Put in SS
	mov	sp, offset DGROUP:tos 	; Load the TOS in SP
	assume	ss:dgroup

;
;	Prepare the segment registers for initialization.  The initialized
;	data is assumed to have been located in the class ROMDATA with begins
;	with the segment of the same name.
;
	mov	ax, dgroup		; The destination for initialized data
	mov	es, ax
	mov	ax, brd			; The source segment
	mov	ds, ax

;
;	Copy the initialized data from its position in ROM to the target
;	address in RAM
;
	mov	si, offset dgroup:idata	; Get the starting offset of the DATA segment
	mov	di, si			; Same offset, different segment
	mov	cx, offset dgroup:bdata	; Get the ending offset (start of BSS)
	sub	cx, di			; Subtract the two for a length
	jcxz	$$1			; Skip if zero length
	shr	cx, 1			; Convert to words
	rep	movsw			; Copy initialized data from ROM to RAM
$$1:

;
;	Copy the FAR_DATA class.  This class can span multiple segments so
;	the copying from ROM to RAM must be iterated as many times as
;	necessary.  We start by making a normalized pointer to the
;	FAR_DATA class in RAM.
;
	mov	cx, far_data		; Get the class base address
	mov	bx, offset bfdata
	mov	dx, efd			; Get the class end address
	mov	ax, offset efdata
	push	cx			; Push the begin pointer
	push	bx
	push	dx			; Push the end pointer
	push	ax
IF	@CodeSize EQ  MM_NEAR
	call	_aNahdiff		; Subtract, difference in DX:AX
ELSE
	call	_aFahdiff
ENDIF
	mov	bx, far_data		; Get the class base address
	mov	es, bx
	mov	di, offset bfdata	; ES:DI destination address

	mov	bx, brfd		; Get the class base address
	mov	ds, bx
	mov	si, offset brfdata	; DS:SI ROM far data source address

	or	dx, dx			; Test if less than a segment
	jz	$$3			; Skip if less than 64K

$$2:
	mov	cx, 08000h		; Move 64KB
	rep	movsw			; Copy a full segment
	mov	cx, ds			; Get the source segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	ds, cx			; Save it back
	mov	cx, es			; Get the destination segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	es, cx			; Save it back
	dec	dx			; Bump the count
	jnz	$$2			; Repeat for all complete segments

$$3:
	mov	cx, ax			; Get the remaining count
	jcxz	$$4			; Skip the copy if zero
	shr	cx, 1			; Convert from bytes to words
	rep	movsw			; Copy the remaining data
$$4:

;
;	Setup the segment registers for calling the Microsoft C 6.0 main()
;
	mov	ax, dgroup
	mov	ds, ax
	mov	es, ax
	assume	ds:dgroup, es:dgroup

;
;	Zero out the near BSS area
;
	xor	ax, ax			; Use a zero fill pattern
	mov	di, offset dgroup:bdata	; Get the start of the BSS class
	mov	cx, offset dgroup:edata	; Get the end address
	sub	cx, di			; Subtract the two for a length
	jcxz	$$30			; Skip if zero
	shr	cx, 1			; Convert to the number of words
	rep	stosw			; Zero out the BSS
$$30:

;
;	Zero out the far BSS area.  This is more complicated since the
;	FARBSS class can span multiple segments.
;
	mov	cx, far_bss		; Get the class base address
	mov	bx, offset bfbss
	mov	dx, efb			; Get the class end address
	mov	ax, offset efbss
	push	cx			; Push the begin pointer
	push	bx
	push	dx			; Push the end pointer
	push	ax
IF	@CodeSize EQ  MM_NEAR
	call	_aNahdiff		; Subtract, difference in DX:AX
ELSE
	call	_aFahdiff
ENDIF
	mov	bx, ax			; DX:BX is the length
	mov	ax, far_bss		; Get the class base address
	mov	es, ax
	mov	di, offset bfbss	; ES:DI destination address

	xor	ax, ax			; Use a zero fill pattern
	or	dx, dx			; Test if less than a segment
	jz	$$32			; Skip if less than 64K

$$31:
	mov	cx, 08000h		; Move 64KB
	rep	stosw			; Clear the segment
	mov	cx, es			; Get the destination segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	es, cx			; Save it back
	dec	dx			; Bump the count
	jnz	$$31			; Repeat for all complete segments

$$32:
	mov	cx, bx			; Get the remaining count
	jcxz	$$33			; Skip the copy if zero
	shr	cx, 1			; Convert from bytes to words
	rep	stosw			; Clear the segment
$$33:

;
;	Call any near and far initializers linked in
;
	mov	si, offset DGROUP:xifbegin
	mov	di, offset DGROUP:xifend
IF	@CodeSize EQ MM_NEAR
	call	farinits
ELSE
	call	inits			; Execute any initializers
ENDIF

	mov	si, offset DGROUP:xibegin
	mov	di, offset DGROUP:xiend
	call	inits			; Execute any initializers

;
;	Call the C language entry point
;
	sti				; Enable interrupts
	call	main			; Call main()

IFNDEF	TDREM				; ROMable code branch
	jmp	_startup		; Application dependent
ELSE
;
;	Hand over control to the exit handler, passing any exit code returned
;	by the application.
;
	and	ax, 7fh			; Mask for TDREM codes
	push	ax
	xor	bx, bx			; Address the interrupt vector table
	mov	es, bx
	pushf				; Push the PSW
	push	bx			; IP
	push	bx			; CS
	jmp	dptr es:[000ch]		; Call the TDREM break handler
ENDIF	; TDREM
EndProc	_startup

;
;	This code installs the initializers linked in the reverse order
;
inits	proc	near
	cmp	si, di			; Test if complete
	jae	alldone

IF	@CodeSize EQ MM_NEAR		; Near data models
	sub	di, 2
	mov	cx, [di]		; Get the initializer address
	jcxz	inits			; Skip the null procedures
	call	cx			; Make the call
ELSE					; Far data models
	sub	di, 4
	mov	ax, [di]
	or	ax, [di+2]
	jz	inits			; Skip any null procedures
	call	dptr [di]		; Call the initializer
ENDIF	; MM_NEAR
	jmp	inits			; Keep looping til done
alldone:
	ret
inits	endp

IF	@CodeSize EQ MM_NEAR
;
;	Only needed in the near code models since the default version will
;	use far initializers automatically.
;
farinits	proc	near
	cmp	si, di			; Test if complete
	jae	falldone
	sub	di, 4
	mov	ax, [di]
	or	ax, [di+2]
	jz	farinits		; Skip any null procedures
	call	dptr [di]		; Call the initializer
	jmp	farinits		; Keep looping til done
falldone:
	ret
farinits	endp
ENDIF	; MM_NEAR

_text	ends

	subttl	Data declarations
	page
_data	segment
public	idata
idata	label	byte			; Mark the start of the DATA class
null	dw	055aah			; NULL address (within the segment)
_data	ends

;
;	Define the initializers.  The main purpose is to define symbols which
;	can be referenced to compute the number of initializers linked in at
;	run-time.
;
xib	segment				; Start of near initializers
xibegin	label	byte
xib	ends
xie	segment				; End of near initializers
xiend	label	byte
xie	ends
xifb	segment				; Start of far initializers
xifbegin label	byte
xifb	ends
xife	segment				; End of far initializers
xifend	label	byte
xife	ends

_bss	segment
public	bdata
bdata	label	byte			; Mark the start of the BSS class
					; and the end of the initialized data
_bss	ends

far_data segment			; BFD - Begin Far Data
bfdata	label	byte			; Mark the start of the FAR_DATA class
far_data ends
efd	segment				; EFD - End Far Data
efdata	label	byte			; Mark the end of the FAR_DATA class
	db	16 dup (?)		; Force the next segment to a new frame
efd	ends

far_bss	segment				; BFB - Begin Far BSS
bfbss	label	byte			; Mark the start of the FAR_BSS class
far_bss	ends
efb	segment				; EFB - End Far BSS
efbss	label	byte			; Mark the end of the FAR_BSS class
	db	16 dup (?)		; Force the next segment to a new frame
efb	ends

;
;	The following segments are used to mark the place for ROM copies
;	of initialized data for use by the startup code.
;
brd	segment				; BRD - Begin ROM Data
brdata	label	byte			; Mark the start of the ROMDATA class
brd	ends
erd	segment				; ERD - End ROM Data
	db	16 dup (?)		; Force the next segment to a new frame
erd	ends

brfd	segment				; BRFD - Begin ROM Far Data
brfdata	label	byte			; Mark the start of the ROMFARDATA class
brfd	ends
erfd	segment				; ERFD - End ROM Far Data
	db	16 dup (?)		; Force the next segment to a new frame
erfd	ends

stack	segment				; Stack start, end of BSS class
public	edata, tos
edata	label	word
IFDEF	STKSIZE
	db	STKSIZE dup (?)
ELSE
	db	2048 dup (?)
	SHOW	<No stack size specified: stack size set to 2KB>
ENDIF	; STKSIZE
	even				; Force word alignment
tos	label	word			; Define the top of stack (the initial SP value)
stack	ends

	end	_startup
