/*
//		Sample application to test dynamic memory allocation by randomly
//		allocated and freeing random sized blocks of memory using the
//		different Microsoft heaps.
*/

#include	<stdlib.h>
#include	<malloc.h>
#include	<string.h>


#define	random(x)	(rand() % (x))
#define	dim(x)		(sizeof(x)/sizeof(x[0]))
#define	ALLOCS		500				/* Number of allocations */
#define	BLOCK_SIZE	200				/* Maximum block size */

char	near *np[ALLOCS] ;				/* Allocated memory block pointers */
char	far *fp[ALLOCS] ;					/* Allocated memory block pointers */


void		main(void)
{
	unsigned	int	item ;				/* Index to block handle */
	unsigned	int	size ;				/* Requested block size */
	unsigned	long	nallocs ;			/* Number of successful allocations */
	unsigned long	nerrors ;			/* Number of failed allocations */
	unsigned	long	fallocs ;			/* Number of successful allocations */
	unsigned long	ferrors ;			/* Number of failed allocations */
	int				heapstatus ;
	unsigned long	nheaperrors ;		/* Number of heap checks failed */
	unsigned long	fheaperrors ;		/* Number of heap checks failed */
	unsigned int	nheapok ;			/* Number of heap checks succeeded */
	unsigned int	fheapok ;			/* Number of heap checks succeeded */
	char				near *pLastNear = NULL ;
	char				far *pLastFar = NULL ;


	/* This loop runs forever allocating and freeing blocks of memory */
	for (;;)   {
		for (item = 0; item < dim(np); item++)   {
			np[item] = NULL ;
			fp[item] = NULL ;
		}

		fheaperrors = nheaperrors = nallocs = fallocs = nerrors = ferrors = 0 ;
		nheapok = fheapok = 0 ;
		for (; !(fheaperrors || nheaperrors);)   {
			/* Compute the pointer and allocation size for the near heap */
			item = random(ALLOCS) ;
			size = random(BLOCK_SIZE) + 1 ;

			/* Free up the memory if in use */
			if (np[item] != NULL)
				_nfree(np[item]) ;

			/* Allocate a new block from the near heap */
			if ((np[item] = _nmalloc(size)) == NULL)   {
				nerrors++ ;
			}
			else   {
				nallocs++ ;
				memset(np[item], item, size) ;
				if (pLastNear < np[item])
					pLastNear = np[item] ;
			}

			/* Compute the pointer and allocation size for the near heap */
			item = random(ALLOCS) ;
			size = random(BLOCK_SIZE * 4) + 1 ;

			/* Free up the memory if in use */
			if (fp[item] != NULL)
				_ffree(fp[item]) ;

			/* Allocate a new block from the far heap */
			if ((fp[item] = _fmalloc(size)) == NULL)   {
				ferrors++ ;
			}
			else   {
				fallocs++ ;
				_fmemset(fp[item], item, size) ;
				if (pLastFar < fp[item])
					pLastFar = fp[item] ;
			}

			/* Check the near heap integrity */
			heapstatus = _nheapchk() ;
			switch (heapstatus)   {
				case _HEAPEMPTY:
				case _HEAPOK:
					nheapok++ ;
					break ;
				default:
					nheaperrors++ ;
					break ;
			}

			/* Check the far heap integrity */
			heapstatus = _fheapchk() ;
			switch (heapstatus)   {
				case _HEAPEMPTY:
				case _HEAPOK:
					fheapok++ ;
					break ;
				default:
					fheaperrors++ ;
					break ;
			}
		}

		/* Free up all memory allocated */
		for (item = 0; item < dim(np); item++)   {
			if (np[item] != NULL)
				_nfree(np[item]) ;
			if (fp[item] != NULL)
				_ffree(fp[item]) ;
		}
	}
}
