/*
//		Microsoft C/C++ 7.0 MS-DOS Int 21h Emulation
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the code for serving common MS-DOS requests from
//		the Microsoft C/C++ 7.0 run-time library.
*/

#include	<stdlib.h>						/* _osmajor, _osminor prototype */
#include	<dos.h>							/* enable(), disable() prototypes */

#include	"typedefs.h"					/* General purpose type declarations */
#include	"dosemu.h"						/* Int 21h function/data prototypes */

#pragma	optimize("geln", off)		/* Disable unsafe optimizations */


/*
//		Function:	Int21Handler	
//
//		This interrupt handler services any Int 21h calls from the application.
//		Since every application will have different requirements, the scope of this
//		handler will vary.  As implemented here, sufficient functionality exists
//		for supporting floating point arithmetic and console I/O.
*/

void	INTERRUPT	Int21Handler(
			UINT		es,	UINT		ds,
			UINT		di,	UINT		si,	UINT		bp,	UINT		sp,
			UINT		bx,	UINT		dx,	UINT		cx,	UINT		ax,
			UINT		ip,	UINT		cs,	UINT		psw
		)
{
	UINT	FAR	*wp ;						/* General purpose UINT pointer */


	/* Permit interrupts while in the Int 21h emulation */
	_enable() ;

	/* Clear the carry to indicate success */
	psw &= 0xfffe ;

	/* The contents of the AH register is used to determine the service */
	switch (highbyte(ax))   {
		case DOS_SETVECTOR:
			wp = (UINT FAR *) MK_FP(0, lowbyte(ax) << 2) ;
			_disable() ;
			*wp++ = dx ;
			*wp = ds ;
			_enable() ;
			break ;
		
		case DOS_VERSION:
			ax = (_osminor << 8) | _osmajor ;
			break ;

		case DOS_GETVECTOR:
			wp = (UINT FAR *) MK_FP(0, lowbyte(ax) << 2) ;
			_disable() ;
			bx = *wp++ ;
			es = *wp ;
			_enable() ;
			break ;

		default:		 						/* All other functions */
#if	defined(TDREM)
			/* Exit with the DOS subfunction as the exit code */
			exit(highbyte(ax)) ;
#endif	/* defined(TDREM) */
			psw |= 1 ;						/* Set the carry flag */
			break ;
	}
}


/*
//		Function:	Int07Handler	
//
//		This handler needs to simply add the size of the ESC instruction
//		to the IP on the stack and return, effectively skipping the
//		instruction.  This is a necessary precaution since depending on the
//		processor used, some 8087 instructions may be executed to test for
//		the presence of a coprocessor (typically a store status) and cause
//		an exception, or worse, lock up the system.  It is recommended that
//		custom floating point initialization code be used to have complete
//		control over floating point operations.
//
//		This support is needed by the 80C186XL/EA/EB/EC and NEC V25/V35-family
//		microprocessors.
*/

void	INTERRUPT	Int07Handler(
			UINT		es,	UINT		ds,
			UINT		di,	UINT		si,	UINT		bp,	UINT		sp,
			UINT		bx,	UINT		dx,	UINT		cx,	UINT		ax,
			UINT		ip,	UINT		cs,	UINT		psw
		)
{
	UCHAR	FAR	*pOpcode ;				/* UCHAR pointer to the opcode */


	/* Make a pointer to the opcode */
	pOpcode = (UCHAR FAR *) MK_FP(cs, ip + 1) ;

	/* Decode the instruction causing the trap and compute the length */
	switch (*pOpcode & 0xc0)   {
		case 0x00:
			ip += ((*pOpcode & 7) == 6) ? 4 : 2 ;
			break ;
		case 0x40:
			ip += 3 ;
			break ;
		case 0x80:
			ip += 4 ;
			break ;
		case 0xc0:
			ip += 2 ;
			break ;
	}
}


/*
//		Function:	InitInt21h
//
//		This function is responsible for initializing the Paradigm Systems
//		MS-DOS Int 21h emulation package.  This must be the first initializer
//		in the Paradigm list since it defines the initial Int 21h handler
//		while subsequent initializers will chain to this handler should they
//		add their own Int 21h services.
*/

void	FAR	InitInt21h(void)
{
	/* Install the interrupt handlers in the vector table */
	setvect(0x07, Int07Handler) ;
	setvect(0x21, Int21Handler) ;
}

#pragma	data_seg("xip", "DATA")
static	void	FAR	*pInit21h = InitInt21h ;
#pragma	data_seg()
