/*
//		MATHERR.C	Sample error handling routine for extrinsic functions
//						in a ROM environment.  This code should be customized
//						for a particular application and then replace the
//						matherr() function supplied by Microsoft in the
//						applicable run-time library.
//	
//		Copyright (C) 1988,1989,1992 Paradigm Systems.  All rights reserved.
//
//		This version of matherr() has been modified to return a HUGE_VAL that
//		can be held in type float.  This is done to prevent the functions such
//		as exp() and log() from returning the default of HUGE_VAL which will cause
//		a floating point overflow exception when it is assigned to a float
//		variable.
*/

#include	<stdlib.h>
#include	<string.h>
#include	<math.h>
#include	<float.h>


int		matherr(
				struct	exception	*x
			)
{
	int	code = 0 ;		/* 0 if matherr() can't handle the exception */

	switch (x->type)   {
		case DOMAIN:
			if (_stricmp(x->name, "exp") == 0 || _strnicmp(x->name, "log", 3) == 0)   {
				x->retval = FLT_MAX * (x->retval < 0.0 ? -1.0 : 1.0) ;
				code = 1 ;
			}
			else if (_stricmp(x->name, "sqrt") == 0)   {
				x->retval = sqrt(fabs(x->arg1)) ;
				code = 1 ;
			}
			break ;

		case SING:
			if (_stricmp(x->name, "exp") == 0 || _strnicmp(x->name, "log", 3) == 0)   {
				x->retval = FLT_MAX * (x->retval < 0.0 ? -1.0 : 1.0) ;
				code = 1 ;
			}
			break ;

		case OVERFLOW:
			if (_stricmp(x->name, "exp") == 0 || _strnicmp(x->name, "log", 3) == 0)   {
				x->retval = FLT_MAX * (x->retval < 0.0 ? -1.0 : 1.0) ;
				code = 1 ;
			}
			break ;

		case PLOSS:
		case TLOSS:
		case UNDERFLOW:
		default:
			break ;
	}

	/* Return the code showing the exception has been handled */
	return	code ;
}
