/*
//		Microsoft C/C++ 7.0 MS-DOS Dynamic Memory Management Support
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the code for initializing and using the near and
//		far heaps found in the Microsoft C/C++ 7.0 run-time library.
//
//		To use the MSC/C++ heap support in your application, you must specify
//		the paragraph addresses of the end of the far heap.  As implemented,
//		the near heap is part of DGROUP and begins after the stack.
//		Following the near heap is the far heap, but it can be set to zero
//		length by undefining the FHEAPEND macro.  See the file MSC7RTL.ASM
//		for the definition of the near heap.
//
//		This method has the advantage of maximizing the heaps.  Be aware that
//		since no segments are declared to hold the heap space, Paradigm LOCATE
//		is unaware that the heaps exist.  Be extra cautious to make sure that
//		the MAP directives reserve space for each of the heaps.
*/

#pragma	check_stack(off)				// No stack checking
#pragma	warning(disable:4103)		// Disable the warning

#include	<stdlib.h>						/* exit() prototype */
#include	<dos.h>							/* enable(), disable() prototypes */

#include	"typedefs.h"					/* General purpose type declarations */
#include	"msc7dmm.h"						/* Heap management definitions */
#include	"dosemu.h"						/* Int 21h function/data prototypes */


#define	FHEAPEND		0x0000			/* Paragraph address of the far heap end */

#pragma	message("Microsoft C/C++ dynamic memory management support, Version 1.0")
#if	(FHEAPEND == 0x0000)
#pragma	message("No Microsoft C/C++ far heap is currently defined")
#endif	// (FHEAPEND == 0x0000)

static
void (INTERRUPT *pOldInt21)() ;		/* Original Int 21h vector */


/*
//		The starting MCB is defined here and must be paragraph-aligned.
*/

#pragma	data_seg("STUBHEAP", "BSS")
static	MCB	mcb ;
#pragma	data_seg()



/*
//		Function:	Int21Handler	
//
//		This interrupt handler services any Int 21h calls from the application.
//		Since every application will have different requirements, the scope of this
//		hanlder will vary.  As implemented here, sufficient functionality exists
//		for supporting floating point arithmetic and console I/O.
*/

void	INTERRUPT	Int21DmmHandler(
			UINT		es,	UINT		ds,
			UINT		di,	UINT		si,	UINT		bp,	UINT		sp,
			UINT		bx,	UINT		dx,	UINT		cx,	UINT		ax,
			UINT		ip,	UINT		cs,	UINT		psw
		)
{
	MCBP		ma ;							/* Pointer to the current MCB */
	MCBP		nma ;							/* Pointer to the next MCB */
	MCBP		tmp ;							/* A working pointer to an MCB */
	UINT		mcbw ;						/* Application segment identifier */
	UINT		size ;						/* Largest sized allocation block */


	/* Permit interrupts while in the Int 21h emulation */
	_enable() ;

	/* Clear the carry to indicate success */
	psw &= 0xfffe ;

	/* The contents of the AH register is used to determine the service */
	switch (highbyte(ax))   {
		case DOS_ALLOC:
			/* Make a pointer to the MCB chain and find an empty block */
			for (ma = &mcb;;)   {
				/* Scan for an empty block in the list */
				while (ma->id != 'Z' && ma->owner == 1)
					ma = (MCBP) MK_FP(FP_SEG(ma) + ma->size + 1, 0) ;

				/* Check if no space exists */
				if (ma->id == 'Z' && (ma->owner == 1 || ma->size <= bx))   {
					/* Return the size block that can be allocated */
					ax = 8 ;
					bx = (ma->owner == 1) ? 0 : ma->size ;
					psw |= 1 ;
					return ;
				}

				/* Check if sufficient space exists */
				if (ma->size <= bx)
					size = ma->size ;
				else
					break ;
			}

			/* Carve out a suitable sized block of RAM */
			nma = (MCBP) MK_FP(FP_SEG(ma) + bx + 1, 0) ;

			/* Initialize the new smaller MCB */
			nma->id = ma->id ;
			nma->owner = ma->owner ;
			nma->size = ma->size - bx - 1 ;

			/* Return the segment of the first block */
			ma->id = 'M' ;
			ma->owner = 1 ;
			ma->size = bx ;
			ax = FP_SEG(ma) + 1 ;
			break ;

		case DOS_REALLOC:
			/* Make a pointer to the MCB */
			ma = (MCBP) MK_FP(es - 1, 0) ;

			/* Verify we are pointing to an MCB */
			if (ma->id != 'M' && ma->id != 'Z')   {
				/* Bad block identifier - return an error code */
				ax = 9 ;
				psw |= 1 ;
				return ;
			}

			/* Now either shrink or expand it */
			if (bx > ma->size)   {
				/* Check for the last block in the chain */
				if (ma->id == 'Z')   {
					/* Return the size block that can be allocated */
					ax = 8 ;
					bx = 0 ;
					psw |= 1 ;
					return ;
				}

 				/* Expand - is the next block is free/sufficiently large */
				nma = (MCBP) MK_FP(es + ma->size, 0) ;
				if (nma->owner != 0 || (ma->size + nma->size) < bx)   {
					/* Return the size block that can be allocated */
					ax = 8 ;
					bx = ma->size + ((nma->owner == 0) ? nma->size : 0) ;
					psw |= 1 ;
					return ;
				}

				/* Steal the space from the second MCB */
				mcbw = bx - ma->size ;
				ma->size = bx ;

				/* Initialize the new smaller MCB */
				tmp = (MCBP) MK_FP(es + bx, 0) ;
				tmp->id = nma->id ;
				tmp->owner = nma->owner ;
				tmp->size = nma->size - mcbw ;
			}
			else if (bx < ma->size)   {
				/* Release - create a new MCB holding the unallocated memory */
				tmp = (MCBP) MK_FP(es + ma->size, 0) ;
				nma = (MCBP) MK_FP(es + bx, 0) ;
				nma->size = ma->size - bx - 1 ;
				nma->owner = 0 ;
				nma->id = ma->id ;

				/* Shrink the size to the specified number */
				ma->size = bx ;

				/* Combine adjacent blocks if possible */
				if (ma->id != 'Z' && tmp->owner == 0)   {
					nma->size += tmp->size + 1 ;
					nma->id = tmp->id ;
				}

				/* Insert into the chain */
				if (ma->id == 'Z')
					ma->id = 'M' ;
			}

			/* Clear the carry to indicate success */
			psw &= 0xfffe ;
			break ;

		default:
			/* All other functions are passed to the old handler for service */
			_chain_intr(pOldInt21) ;
			break ;
	}
}


/*
//		Function:	InitHeap
//
//		This function is responsible for the initialization of the heap
//		and the DOS memory chains used to support the Microsoft heap
//		routines.
//
//		Two blocks are defined for use by the heap management functions.  The
//		first block is used to hold the contents the near heap and it is
//		owned by the application.  The second block is used for the far heap and
//		is only present when a far heap is defined.
*/

void	FAR	InitHeap(void)
{
	MCBP		ma ;							/* Pointer to the MSC chain */
	extern	UINT	_endnheap ;			/* Paragraph address of near heap end */


	/* Initialize the DOS memory block list */
	ma = &mcb ;
	ma->id = 'M' ;
	ma->owner = 1 ;
	ma->size = _endnheap - FP_SEG(ma) - 1 ;

	/* Initialize the _psp to the start of the near heap */
	_psp = FP_SEG(ma) + 1 ;

#if	(FHEAPEND != 0x0000)
	/* Check that the heap is legal */
	if (_endnheap >= FHEAPEND)
		exit(-11) ;

	/* Now define the far heap memory block */
	ma = (MCBP) MK_FP(FP_SEG(ma) + 1 + ma->size, 0) ;
	ma->id = 'Z' ;
	ma->owner = 0 ;
	ma->size = FHEAPEND - FP_SEG(ma) - 1 ;
#else
	ma->id = 'Z' ;
#endif	//	(FHEAPEND != 0x0000)

	/* Install the heap extension Int 21h handler */
	pOldInt21 = _dos_getvect(0x21) ;
	_dos_setvect(0x21, Int21DmmHandler) ;
}


/*
//		Insert a pointer into the initialization table for the heap
//		initialization routine.  This is placed in the far initializer
//		table to guarantee the default Int 21h handler is already installed.
*/

#pragma	data_seg("xif", "DATA")
static	void	FAR	*pInitHeap = InitHeap ;
#pragma	data_seg()
