/*
//		Extensible MS-DOS Emulation for Microsoft C/C++ Embedded Systems
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the definitions common to modules implementing
//		the Microsoft C/C++ 7.0 heap support.
*/

#if	!defined(_MSCDMM)
#define	_MSCDMM

#pragma	pack(1)							// Force byte packing of structures

/*
//		Define the format of the DOS memory control block (MCB).  Only the
//		id and size fields need be implemented since the all memory is assumed
//		to be used by a single application.  If a multitasking system is built,
//		placing the task id in the owner field allows freeing the memory owned
//		by the task when it exits.  An id field of 0 is used to mark an unused
//		block.
//
//		The layout in memory consists of an MCB followed by the memory block.
//		The allocator returns to the caller the segment of the block which
//		can be decremented to find the corresponding MCB.
*/

typedef	struct   {
		UCHAR		id ;						/* 'M' or 'Z' */
		UINT		owner ;					/* Application PID or PSP */
		UINT		size ;					/* Size in paragraphs */
		UCHAR		pad[11] ;				/* Pad to paragraph length */
	} MCB, far *MCBP ;					/* Structure and pointer definition */


typedef	struct   {
		UINT		checksum ;				/* Checksum area */
		UINT		flags ;					/* Flags word */
		UINT		segsize ;				/* Size of segment */
		UINT		start ;					/* Offset of first heap entry */
		UINT		rover ;					/* Rover offset */
		UINT		last ;					/* Offset to end-of-heap marker */
		ULONG		nextseg ;				/* Far pointer to next _heap_seg_desc */
		ULONG		prevseg ;				/* Far pointer to previous _heap_seg_desc */
	} HEAPREC, far *HEAPRECP ;			/* Heap descriptor definitions */


typedef	enum   {
		_HEAP_MODIFY = 0x01,				/* Heap segment size can be modified */
		_HEAP_FREE = 0x02,				/* Heap segment may be freed up to OS */
		_HEAP_NEAR = 0x04,				/* Heap segment is part of the near heap */
		_HEAP_BASED = 0x08				/* Heap segment is part of the based heap */
	} HFLAGS ;


/*
//		Heap Linked List Descriptor
//
//		Some heap routines make assumptions about the layout of the
//		heap list descriptor.  If this descriptor changes, some routines
//		may no longer work correctly.
*/

typedef	struct   {
		ULONG		startseg	;				/* Pointer to first heap descriptor */
		ULONG		roverseg ;				/* Rover pointer */
		ULONG		lastseg ;				/* Pointer to last heap descriptor */
		UINT		segflags ;				/* Flags word for init'ing new segs */
	} HEAPLIST ;

#endif	/* defined(_MSCDMM) */
