	page	60, 132
	name	msc7rtl
	title	Microsoft C/C++ 7.0 Run-Time Library Support Package

;
;	Microsoft C/C++ 7.0 Run-Time Library Definitions and Initialization
;	Copyright (C) 1992 Paradigm Systems.  All rights reserved.
;
;	Function		
;	========
;	This module is responsible for the installation and initialization
;	for supported Microsoft C/C++ 7.0 run-time library routines.  These
;	functions include embedded system stack checking support, exit and
;	exception handling, and heap initialization.
;

	INCLUDE	startup.inc		; Macros/assembly langauge definitions
	INCLUDE	msc70.inc		; MSC/C++ 7.0 specific definitions

	subttl	Segment ordering/alignment section
	page
;
;	Segment and group declarations.  The order of these declarations is
;	used to control the order of segments in the .EXE file since the
;	Microsoft linker copies segments in the same order they are
;	encountered in the object files.
;

DefSeg	_text,    word,  public,  CODE,	       <>	; Default code
DefSeg	_data,    word,  public,  DATA,        DGROUP	; Initialized data
DefSeg	xiheap,   word,  common,  DATA,        DGROUP	; Heap initialization
DefSeg	_bss,     word,  public,  BSS,         DGROUP	; Uninitialized data
DefSeg	stubheap, para,  public,  BSS,         DGROUP	; Heap stub segment
DefSeg	stack,    para,  stack,   STACK,       DGROUP	; Program stack
DefSeg	nheap,    para,  public,  STACK,       DGROUP	; Near heap start
DefSeg	enheap,   para,  public,  STACK,       DGROUP	; Near heap end
DefSeg	fheap,    para,  public,  STACK,       <>	; Far heap start
DefSeg	efheap,   para,  public,  STACK,       <>	; Far heap end

	page
ExtProc	_startup			; Destination in case of exceptions
extrn	__edata : word			; Offset for the bottom of the stack

_text	segment
	assume	cs:_text, ds:dgroup, ss:dgroup

;
;	Run-time stack checking support
;

BegProc	_chkstk
IF	@CodeSize EQ MM_NEAR
LabelB	_aNchkstk, public
ELSE
LabelB	_aFchkstk, public
ENDIF
;
;	Begin by saving the return address from the stack
;
IF	@CodeSize EQ MM_NEAR
	pop	cx			; Get the return offset
ELSE
	pop	cx			; Get the return offset
	pop	dx			; Get the return segment
ENDIF	; MM_NEAR
	mov	bx, sp			; Get the current stack pointer offset
	sub	bx, ax			; Adjust the stack by the specified amount
	jb	no_space		; Report the error if the sign changed

;
;	See if the bottom of the stack has been reached.  This is checked
;	by comparing the new stack pointer offset with the offset defined
;	in the stack segment declaration.
;
	cmp	bx, [_STKHQQ]		; Check for reaching the bottom of the stack
	jb	no_space		; Report the error

	mov	sp, bx			; Restore the new stack pointer offset

IF	@CodeSize EQ MM_NEAR
	push	cx			; Push the return offset
ELSE
	push	dx			; Push the return segment
	push	cx			; Push the return offset
ENDIF	; MM_NEAR
	ret				; Return - no errors

no_space:
IFDEF	TDREM
	mov	ax, TDE_STKOVL		; TDREM stack overflow error code
	push	ax
	call	exit
ELSE
	; *** THIS IS APPLICATION DEPENDENT ***
	mov	sp, bx			; Restore the new stack pointer offset
IF	@CodeSize EQ MM_NEAR
	pop	cx			; Get the return offset
ELSE
	pop	cx			; Get the return offset
	pop	dx			; Get the return segment
ENDIF	; MM_NEAR
	ret
	; *** THIS IS APPLICATION DEPENDENT ***
ENDIF	; TDREM
EndProc	_chkstk


BegProc	_rchkstk
IF	@CodeSize EQ MM_NEAR
LabelB	_aNrchkstk, public
ELSE
LabelB	_aFrchkstk, public
ENDIF
;
;	Calculate the new SP
;
	push	si
	push	di

	mov	di, sp
	sub	di, cx
	jb	r_no_space		; Report the error if the sign changed

;
;	See if the bottom of the stack has been reached.  This is checked
;	by comparing the new stack pointer offset with the offset defined
;	in the stack segment declaration.
;
	cmp	di, [_STKHQQ]		; Check for reaching the bottom of the stack
	jb	r_no_space		; Report the error

	mov	si, sp
	mov	sp, di
	jcxz	done

	mov	cx, ss
	mov	es, cx
	movsw	wptr es:[di], wptr ss:[si]
	movsw	wptr es:[di], wptr ss:[si]
	movsw	wptr es:[di], wptr ss:[si]

IF	@CodeSize EQ MM_FAR
	movsw	wptr es:[di], wptr ss:[si]
ENDIF	; MM_FAR

done:
	pop	di
	pop	si
	ret

r_no_space:
IFDEF	TDREM
	mov	ax, TDE_STKOVL		; TDREM stack overflow error code
	push	ax
	call	exit
ELSE
	; *** THIS IS APPLICATION DEPENDENT ***
	mov	sp, bx			; Restore the new stack pointer offset
IF	@CodeSize EQ MM_NEAR
	pop	cx			; Get the return offset
ELSE
	pop	cx			; Get the return offset
	pop	dx			; Get the return segment
ENDIF	; MM_NEAR
	ret
	; *** THIS IS APPLICATION DEPENDENT ***
ENDIF	; TDREM
EndProc	_rchkstk


;
;	Function:	_exit()
;
;	The function _exit() gets control and should be modified to
;	take an appropriate action for restarting the system.
;

public	__exit
__exit	proc
IFNDEF	TDREM				; ROMable code branch
	jmp	_startup		; Application dependent
ELSE
	mov	bp, sp			; Prepare the stack frame
IF	@CodeSize EQ MM_NEAR
	mov	ax, [bp+2]		; After IP
ELSE
	mov	ax, [bp+4]		; After CS:IP
ENDIF	; MM_NEAR
	xor	bx, bx			; Address the interrupt vector table
	mov	es, bx

	pushf				; Push the PSW
	push	bx			; IP
	push	bx			; CS
	jmp	dptr es:[000ch]		; Call the TDREM break handler
ENDIF	; TDREM
__exit	endp


;
;	Function:	abort()
;
;	The function abort() is a replacement for the Microsoft RTL function
;	of the same name.
;

BegProc	abort				; RTL abort()
IFDEF	TDREM
	mov	ax, TDE_ABORT		; Abort error code for TDREM
ELSE
	mov	ax, 3			; RTL-compatible abort error code
ENDIF
	push	ax
	call	pptr __exit		; Call the exit routine (no return)
EndProc	abort


;
;	Function:	exit()
;
;	The function exit() is a replacement for the Microsoft RTL function
;	of the same name.
;

BegProc	exit				; RTL exit()
IF	@CodeSize NE MM_NEAR
	pop	cx			; Get rid of CS
ENDIF	; MM_NEAR
	pop	cx			; Clear IP (leave the exit code)
	call	pptr __exit		; Call the exit routine (no return)
EndProc	exit


;
;	Functions:	__NMSG_WRITE
;			__wrt2err
;			__FF_MSGBANNER
;
;	These are defined here to satisfy some non-public Microsoft run-time
;	libraries and hopefully should never be called.  Modify as required
;	by your application.
;

BegProc	_FF_MSGBANNER
BegProc	_NMSG_WRITE
BegProc	_wrt2err
IFNDEF	TDREM
	; *** APPLICATION DEPENDENT ***
	cli				; Stop here till reset
	hlt				; Application dependent
ELSE					; TDREM code
	call	abort			; Unknown fault
ENDIF	; TDREM
EndProc	_wrt2err
EndProc	_NMSG_WRITE
EndProc	_FF_MSGBANNER


;
;	Function:	_fptrap()
;
;	The function _fptrap() is a replacement for the Microsoft RTL function
;	of the same name.
;

BegProc	_fptrap				; Default FP exceptions
IFDEF	TDREM				; TDREM code
	mov	ax, TDE_FPEXCP		; FP exception error code
	push	ax
	call	exit			; Report the exception
ELSE					; ROMable code
	cli				; Stop here till reset
	hlt				; Application dependent
ENDIF	; TDREM
EndProc	_fptrap


;
;	Function:	_hinit()
;
;	This function initializes the heap
;

BegProc	_hinit, near			; Heap initialization
;
;	Pop the return address so the stack can be manipulated
;
	pop	dx

;
;	Compute the size of DGROUP, including the near heap
;
	mov   ax, enheap
	mov   bx, ds
	sub   ax, bx
	mov   cl, 4
	shl   ax, cl
	dec   ax
	mov   _asizds, ax

;
;	Initialize the near heap descriptor stub
;
	mov   _nheap_desc.checksum, ds
	mov   _nheap_desc.flags, _HEAP_NEAR OR _HEAP_MODIFY
	mov   _nheap_desc.segsize, sp
	mov   ax, _HEAP_END
	push  ax
	mov   _nheap_desc.last, sp
	not   ax
	push  ax
	mov   _nheap_desc.start, sp
	mov   _nheap_desc.rover, sp

;
;	Initialize the top of stack location
;
	mov   _atopsp, sp

;
;	Restore the return address and exit
;
	push	dx
	ret
EndProc	_hinit

_text	ends
	
	subttl	Data Declarations
	page

_data	segment
public	_STKHQQ, __aDBdoswp
_STKHQQ		dw	dgroup:__edata
__aDBdoswp	=	0d6d6h

GlobalB _osmajor, 2			; Pretend to be DOS 2.10
GlobalB _osminor, 10
GlobalB _osmode, 0 			; _DOS_MODE

GlobalW	_endnheap, enheap
GlobalW	_endfheap, efheap
_data	ends

xiheap	segment
	dw	_hinit			; Override to heap initialization code
xiheap	ends

_bss	segment
GlobalW	errno				; errno used by RTL
GlobalW	_doserrno			
GlobalW	_child
GlobalB	_osfile

GlobalW	_psp				; Heap management variables
GlobalW	_atopsp
GlobalW	_aseghi
GlobalW	_aseglo

; *** NOTE *** This ordering must be preserved!
GlobalW	_asizds
LabelB	_nheap_desc, public
_heap_seg_desc	<>
; *** NOTE ***
_bss	ends

nheap	segment
LabelW	nearheapstart
NHEAPSIZE	=	8192
	db	NHEAPSIZE dup (?)
nheap	ends
enheap	segment
LabelW	nearheapend
	db	16 dup (?)
enheap	ends

efheap	segment
LabelW	farheapend
	db	16 dup (?)
efheap	ends

	end
