/*
//		Microsoft C/C++ 7.0 MS-DOS Dynamic Memory Management Support
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the code for initializing and using Microsoft
//		C/C++ streams.
*/

#include	<stdio.h>						/* stream definitions */
#include	<stdlib.h>						/* exit() prototype */
#include	<dos.h>							/* get/set interrupt vector prototypes */

#include	"typedefs.h"					/* General purpose type declarations */
#include	"dosemu.h"						/* Int 21h function/data prototypes */


static
void (INTERRUPT *pOldInt21)() ;		/* Original Int 21h vector */

UINT		_nfile = 5 ;					/* Number of supported file handles */

UINT		_cflush = 0 ;					/* Stubs to satisfy the MSC/C++ RTL */
UINT		__aDBswpchk = 0 ;
UINT		__aDBswpflg = 0 ;


/*
//		Function:	_commit
//
//		This is a stub for the RTL routine of the same name
*/

#pragma	check_stack(off)
int	__cdecl	_commit(void)
{
	return	0 ;
}
#pragma	check_stack()


/*
//		Function:	_stbuf
//
//		This is a stub for the RTL routine of the same name
*/

#pragma	check_stack(off)
#if	defined(M_I86MM) || defined(M_I86LM) || defined(M_I86HM)
int	__cdecl	__near	__based	(__segname("_TEXT"))	_stbuf(
				FILE		*str
			)
#else
int	__cdecl	_stbuf(
				FILE		*str
			)
#endif
{
	return	1 ;
}
#pragma	check_stack()


/*
//		Function:	_ftbuf
//
//		This is a stub for the RTL routine of the same name
*/

#pragma	check_stack(off)
#if	defined(M_I86MM) || defined(M_I86LM) || defined(M_I86HM)
int	__cdecl	__near	__based	(__segname("_TEXT"))	_ftbuf(
				int		flag,
				FILE		*str
			)
#else
int	__cdecl	_ftbuf(
				int		flag,
				FILE		*str
			)
#endif
{
}
#pragma	check_stack()


/*
//		Function:	Int21SioHandler	
//
//		This interrupt handler services any Int 21h calls from the application.
//		Since every application will have different requirements, the scope of this
//		hanlder will vary.  As implemented here, sufficient functionality exists
//		for supporting floating point arithmetic and console I/O.
*/

void	INTERRUPT	Int21SioHandler(
			UINT		es,	UINT		ds,
			UINT		di,	UINT		si,	UINT		bp,	UINT		sp,
			UINT		bx,	UINT		dx,	UINT		cx,	UINT		ax,
			UINT		ip,	UINT		cs,	UINT		psw
		)
{
	UINT		i ;							/* General purpose counter */
	CHAR		FAR *cp ;					/* General purpose characater pointer */
	int		c ;							/* Temporary character storage */


	/* Permit interrupts while in the Int 21h emulation */
	_enable() ;

	/* Clear the carry to indicate success */
	psw &= 0xfffe ;

	/* The contents of the AH register is used to determine the service */
	switch (highbyte(ax))   {
		case DOS_READ:
			cp = (CHAR FAR *) MK_FP(ds, dx) ;
			for (i = 0; i < cx; )   {
				c = _getch() ;
				/* Check for and process backspace characters */
				if (c == '\b' && i > 0)   {
					_putch(c) ;
					_putch(' ') ;
					_putch(c) ;
					cp-- ;
					i-- ;
					continue ;
				}
				/* Check for the end of the line */
				if (c == '\r')   {
					*cp++ = _putch('\r') ;
					*cp = _putch('\n') ;
					i += 2 ;
					break ;
				}
				/* Put the character in the buffer */
				_putch(*cp++ = c) ;
				i++ ;
			}

			/* Clear the carry to indicate success */
			psw &= 0xfffe ;

			/* Return the number of characters read in AX */
			ax = i ;
			break ;

		case DOS_WRITE:
			cp = (CHAR FAR *) MK_FP(ds, dx) ;
			for (i = 0; i < cx; i++)   {
				/* Check for and handle a backspace charaacter */
				if (*cp == '\b')   {
					_putch(*cp) ;
					_putch(' ') ;
				}
				_putch(*cp++) ;
			}

			/* Clear the carry to indicate success */
			psw &= 0xfffe ;

			/* Return the number of characters written in AX */
			ax = i ;
			break ;

		case DOS_SEEK:
			/* Just ignore for console streams */
			break ;

		case DOS_IOCTL:
			if (lowbyte(ax) == 0)   {
				dx = 0x80d3 ;
			}
			else   {
				psw |= 1 ;					/* Signal an error */
				ax = 6 ;						/* Invalid or not open handle */
			}
			break ;

		default:
			/* All other functions are passed to the old handler for service */
			_chain_intr(pOldInt21) ;
			break ;
	}
}


/*
//		Function:	InitStreams
//
//		This function is responsible for the initialization of the stream
//		I/O package.
*/

void	FAR	InitStreams(void)
{
	/* Install the stream extension Int 21h handler */
	pOldInt21 = _dos_getvect(0x21) ;
	_dos_setvect(0x21, Int21SioHandler) ;

	/* Initialize the stream hardware */
	InitConsole() ;

	/* Assign the stream buffering */
	setvbuf(stdout, NULL, _IONBF, 0) ;
}


/*
//		Insert a pointer into the initialization table for the stream
//		initialization routine.  This is placed in the far initializer
//		table to guarantee the default Int 21h handler is already installed.
*/

#pragma	data_seg("xif", "DATA")
static	void	FAR	*pInitStreams = InitStreams ;
#pragma	data_seg()
