/*
//		TYPEDEFS.H - General purpose definition file
//		Copyright (C) 1992 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the general purpose definitions common to the
//		all Paradigm applications.  By defining synonyms for the physical
//		data types to be manipulated, portability between memory models
//		and machines is maximized.
//
//		Note that this file follows the system include files and before
//		any application include files.
*/

#if	!defined(_TYPEDEFS)
#define	_TYPEDEFS

/*
//		Define the types to be used to manipulate 8-, 16-, 32-bit data.
*/

typedef	unsigned char	BIT ;			/* Use for defining MSC bit fields */

typedef	char				CHAR ;		/* 8-bit signed data */
typedef	unsigned char	UCHAR ;		/* 8-bit unsigned data */
typedef	int				INT ;			/* 16-bit signed data */
typedef	unsigned int	UINT ;		/* 16-bit unsigned data */
typedef	long				LONG ;		/* 32-bit signed data */
typedef	unsigned long	ULONG ;		/* 32-bit unsigned data */

typedef	float				FLOAT ;		/* 32-bit floating point data */
typedef	double			DOUBLE ;		/* 64-bit floating point data */

#define	NEAR				__near		/* Shortcuts */
#define	FAR				__far
#define	INTERRUPT		__cdecl __interrupt __far

#if	defined(__cplusplus)
typedef	void (INTERRUPT *ISRP)(...) ;	/* Interrupt table vector entries */
#else
typedef	void (INTERRUPT *ISRP)() ;		/* Interrupt table vector entries */
#endif

/*
//		TDREM application error/exit codes
*/

typedef	enum   {
		TDE_DMMERR = 243,					/* heap initialization error */
		TDE_FPEXCP,							/* floating point exception */
		TDE_STKOVL,							/* stack overflow */
		TDE_ABORT,							/* abort() */
	}	TDERRS ;


/*
//		Define some common macros that show up everywhere.  These are
//
//			lowbyte(x)		Extract the low order byte of UINT x
//			highbyte(x)		Extract the high order byte of UINT x
//			dim(x)			Computes the dimension of an array
*/

#define	lowbyte(word)		((word) & 0xff)
#define	highbyte(word)		lowbyte((word) >> 8)
#define	dim(x)				(sizeof(x) / sizeof(x[0]))

#if	!defined(min)
#define max(a,b)	(((a) > (b)) ? (a) : (b))
#define min(a,b)	(((a) < (b)) ? (a) : (b))
#endif

/*
//		This macro is used to install an interrupt vector entry directly.
*/

#define	setvect(inum, addr) *((ISRP far *) ((inum) * 4)) = ((ISRP) addr)
#define	MK_FP(seg,ofs)	((void __far *) (((ULONG)(seg) << 16) | (UINT)(ofs)))

#endif	/* !defined(_TYPEDEFS) */
