	page	60, 132
	name	msc70
	title	Microsoft C/C++ 7.0 Startup Code, Version 1.00

;
;	Embedded System Startup Code for Microsoft C/C++ 7.0
;	Copyright (C) 1992 Paradigm Systems.  All rights reserved.
;
;	Make sure that this startup module is specified first when linking
;	the application.  Requires MASM 5.1 or MASM 6.0 to assemble.
;
;		masm /mx /D__MDL__ msc70.asm ;
;		ml /c /Cx /D__MDL__ msc70.asm ;
;
;	where
;		__MDL__ is one of the following:
;
;		__S__	Small memory model
;		__C__	Compact memory model
;		__M__	Medium memory model
;		__L__	Large memory model
;		__H__	Huge memory model
;
;	If no memory model is specified, the small memory model is
;	assumed and a warning is issued.
;
;	This module is complete in that it supports all memory models for
;	either ROMable or TDREM applications.  Being a "worst case"
;	construction, this startup can be shortened considerably if the
;	application does not make use of the FAR_BSS or FAR_DATA classes.
;
	
	INCLUDE	startup.inc		; Macros/assembly language definitions
	INCLUDE	msc70.inc		; MSC/C++ 7.0 specific definitions

	SHOW	<Paradigm LOCATE Microsoft C/C++ 7.0 Startup Support>
IFDEF	TDREM
	SHOW	<Building Paradigm TDREM-compatible startup code>
ENDIF	;	TDREM

	subttl	Segment ordering/alignment section
	page
;
;	Segment and group declarations.  The order of these declarations is
;	used to control the order of segments in the .EXE file since the
;	Microsoft linker copies segments in the same order they are
;	encountered in the object files.
;

DefSeg	_text,    para,  public,  CODE,	       <>	; Default code


;
;	These segments are only used when the application contains large
;	initialized or uninitialized data structures or uses huge data.
;

DefSeg	_fdata,   para,  public,  FAR_DATA,    <>	; Far initialized data
DefSeg	_efdata,  para,  public,  ENDFAR_DATA, <>	
DefSeg	_fbss,    para,  public,  FAR_BSS,     <>	; Far uninitialized data
DefSeg	_efbss,	  para,  public,  ENDFAR_BSS,  <>


;
;	Define the segments that will form the group DGROUP.  The order of
;	these classes must not be changed since some of the startup code
;	assumes a specific relationship between classes.
;

DefSeg	_data,    para,  public,  DATA,        DGROUP	; Initialized data
DefSeg	xiheap,   word,  common,  DATA,        DGROUP	; Heap initialization
DefSeg	const,    word,  public,  CONST,       DGROUP	; Constant data
DefSeg	hdr,      word,  public,  MSG,         DGROUP	; Misc. data
DefSeg	_bss,     word,  public,  BSS,         DGROUP	; Uninitialized data
DefSeg	stack,    para,  stack,   STACK,       DGROUP	; Program stack


;
;	The following segments are used to mark the place for ROM copies
;	of initialized data for use by the startup code.
;

DefSeg	_brd,     para,  public,  ROMDATA,       <>	; Copy of initialized data
DefSeg	_erd,     para,  public,  ENDROMDATA,    <>
DefSeg	_brfd,    para,  public,  ROMFARDATA,    <>	; Copy of far initialized data
DefSeg	_erfd,    para,  public,  ENDROMFARDATA, <>


;
;	Define functions/procedures referenced here but defined elsewhere.
;	main() is the most recognized function but we also execute C/C++
;	initializers/constructors and some run-time library helpers.
;

ExtProc	main				; User application entry point
ExtProc	_cinit				; C/C++ initializers/constructors
IF	@CodeSize EQ MM_NEAR
ExtProc	_aNahdiff, near			; Specify the huge pointer difference
diffProc	equ	_aNahdiff	; function name
ELSE
ExtProc	_aFahdiff, far			; Specify the huge pointer difference
diffProc	equ	_aFahdiff	; function name
ENDIF	; MM_NEAR
		
public	__acrtused			; Object modules reference this symbol
	__acrtused = 9876h		; to pull in startup code
public	__aDBused			; Created when the /qc option is used
	__aDBused = 0d6d6h

	subttl	Startup/initialization code
	page
_text	segment
	assume	cs:_text

BegProc	_startup, far
IFDEF	TDREM
;
;	Check if the segment registers have been setup by DOS.  TDREM always
;	clears the DS/ES segment registers so it can be used to check if the
;	application is accidently running under MS-DOS instead of TDREM.
;
	mov	ax, es
	or	ax, ax
	jz	embedded

	push	cs			; Display a DOS error message
	pop	ds
	assume	ds:_text
	mov	dx, offset errmsg
	mov	ah, 09h
	int	21h
	mov	ah, 04ch		; Exit before doing something harmful
	int	21h			; No return from this DOS call
errmsg	db	"Embedded application: DOS not supported", 0ah, 0dh, '$'
embedded:
ENDIF	; TDREM

	cli				; Disable interrupts
	cld				; Make sure the direction is forward

;
;	Initialize the stack segment and pointer registers to point to the
;	default stack.
;
	mov	ax, dgroup
	mov	ss, ax
	mov	sp, offset dgroup:_tos
	assume	ss:dgroup

;
;	Prepare the segment registers for initialization.  The initialized
;	data is assumed to have been located in the class ROMDATA which begins
;	with the segment _BRD.
;
	mov	ax, dgroup
	mov	es, ax
	mov	ax, _brd
	mov	ds, ax

;
;	Copy the initialized data from its position in ROM to the target
;	address in RAM.  This is done by subtracting the starting offset
;	of the initialized data from the start of the BSS data to compute
;	the length of the copy.  Because this is a group, there is never more
;	than 64K of data to copy.
;
	mov	si, offset dgroup:idata
	mov	di, si
	mov	cx, offset dgroup:bdata
	sub	cx, di
	jcxz	@F
	shr	cx, 1
	rep	movsw
@@:

;
;	Copy the FAR_DATA class.  This class can span multiple segments so
;	the copying from ROM to RAM must be iterated as many times as
;	necessary if the class is larger than 64KB in size.  We start by
;	making a normalized pointer to the FAR_DATA class in RAM.  To save
;	time and space, we use a run-time library helper function to compute
;	difference between the huge pointers.
;
	mov	cx, _fdata		; Get the class base address
	mov	bx, offset bfdata
	mov	dx, _efdata		; Get the class end address
	mov	ax, offset efdata
	push	cx			; Push the begin address pointer
	push	bx
	push	dx			; Push the end address pointer
	push	ax
	call	diffProc		; Compute the difference
	mov	bx, _fdata		; Get the class base address
	mov	es, bx
	mov	di, offset bfdata	; ES:DI destination address

	mov	bx, _brfd		; Get the class base address
	mov	ds, bx
	mov	si, offset brfdata	; DS:SI ROM far data source address

	or	dx, dx			; Test if less than a segment
	jz	not64K			; Skip if less than 64K

@@:
	mov	cx, 08000h		; Move 64KB
	rep	movsw			; Copy a full segment
	mov	cx, ds			; Get the source segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	ds, cx			; Save it back
	mov	cx, es			; Get the destination segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	es, cx			; Save it back
	dec	dx			; Bump the count
	jnz	@B			; Repeat for all complete segments

not64K:
	mov	cx, ax			; Get the remaining count
	jcxz	@F			; Skip the copy if zero
	shr	cx, 1			; Convert from bytes to words
	rep	movsw			; Copy the remaining data
@@:

;
;	Setup the segment register conventions that will be used to call the
;	Microsoft C/C++ 7.0 main() function.
;
	mov	ax, dgroup
	mov	ds, ax
	mov	es, ax
	assume	ds:dgroup, es:dgroup

;
;	Zero out the near BSS area
;
	xor	ax, ax			; Use a zero fill pattern
	mov	di, offset dgroup:bdata	; Get the start of the BSS class
	mov	cx, offset dgroup:_edata; Get the end address
	sub	cx, di			; Subtract the two for a length
	jcxz	@F			; Skip if zero
	shr	cx, 1			; Convert to the number of words
	rep	stosw			; Zero out the BSS
@@:

;
;	Zero out the far BSS area.  This is more complicated since the
;	FARBSS class can span multiple segments, exceeding 64K in length.
;
	mov	cx, _fbss		; Get the class base address
	mov	bx, offset bfbss
	mov	dx, _efbss		; Get the class end address
	mov	ax, offset efbss
	push	cx			; Push the begin pointer
	push	bx
	push	dx			; Push the end pointer
	push	ax
	call	diffProc		; Compute the difference
	mov	bx, ax			; DX:BX is the length
	mov	ax, _fbss		; Get the class base address
	mov	es, ax
	mov	di, offset bfbss	; ES:DI destination address

	xor	ax, ax			; Use a zero fill pattern
	or	dx, dx			; Test if less than a segment
	jz	$$64			; Skip if less than 64K

@@:
	mov	cx, 08000h		; Clear a full 64KB segment
	rep	stosw			; Clear the segment
	mov	cx, es			; Get the destination segment
	add	cx, 1000h		; Adjust for the 64K just transfered
	mov	es, cx			; Save it back
	dec	dx			; Bump the count
	jnz	@B			; Repeat for all complete segments

$$64:
	mov	cx, bx			; Get the remaining count
	jcxz	@F			; Skip the copy if zero
	shr	cx, 1			; Convert from bytes to words
	rep	stosw			; Clear the segment
@@:

;
;	Near/far heap initialization is performed here since the stack is
;	modified to initialize the near heap.  This guarantees that only one
;	level of function nesting is active when the initializer is called.
;
	mov	cx, [xinheap]
	jcxz	@F
	call	word ptr [xinheap]
@@:

;
;	Execute any of C/C++ initializers that are present
;
	xor	bp, bp
	call	_cinit

;
;	Call the C/C++ language entry point main() without any parameters.
;	Arguments can be passed to main() by pushing them on the stack before
;	making the call.
;
	sti				; Interrupts enabled from now on
	call	main			; Turn over control to the user application

;
;	A return from main() is application dependent.  If TDREM is being used,
;	we return the contents of AX as the application exit code; otherwise,
;	the user must decide how to handle this condition.
;
IFNDEF	TDREM
	jmp	_startup		; Application dependent - MODIFY
ELSE
;
;	Hand over control to the exit handler, passing any exit code returned
;	by the application.
;
	and	ax, 7fh			; Mask for TDREM codes
	push	ax
	xor	bx, bx			; Address the interrupt vector table
	mov	es, bx
	pushf				; Push the PSW
	push	bx			; IP is zero
	push	bx			; CS is also zero
	jmp	dptr es:[000ch]		; Call the TDREM break handler
ENDIF	;	TDREM
EndProc	_startup

GlobalW	_dataseg, dgroup		; Used to access DGROUP value

_text	ends

	subttl	Data declarations
	page
_data	segment
public	idata
idata	label	byte			; Mark the start of the DATA class
null	dw	055aah			; NULL address (within the segment)
_data	ends

xiheap	segment
xinheap	label	word			; Near heap initializer offset
	dw	0			; Override via linked in module
xiheap	ends

_bss	segment
public	bdata
bdata	label	byte			; Mark the start of the BSS class
_bss	ends

_fdata segment
bfdata	label	byte			; Mark the start of the FAR_DATA class
_fdata ends
_efdata	segment
efdata	db	16 dup (?)		; Mark end and force a new segment frame
_efdata	ends

_fbss	segment
bfbss	label	byte			; Mark the start of the FAR_BSS class
_fbss	ends
_efbss	segment
efbss	db	16 dup (?)		; Mark end and force a new segment frame
_efbss	ends

;
;	The following segments are used to mark the place for ROM copies
;	of initialized data for use by the startup code.  We need two sets of
;	classes, a pair to hold the place of the near initialized data and
;	another pair to hold the place of the for the far initialized data.
;
_brd	segment
brdata	label	byte			; Mark the start of the ROMDATA class
_brd	ends
_erd	segment
	db	16 dup (?)		; Force the next segment to a new frame
_erd	ends

_brfd	segment
brfdata	label	byte			; Mark the start of the ROMFARDATA class
_brfd	ends
_erfd	segment
	db	16 dup (?)		; Force the next segment to a new frame
_erfd	ends

;
;	The stack segment differs from the Microsoft DOS startup code in that
;	the stack is assigned a fixed length.
;
stack	segment
LabelW	_edata, public
IFDEF	STKSIZE
	db	STKSIZE dup (?)
ELSE
	db	2048 dup (?)
	SHOW	<No stack size specified: stack size set to 2KB>
ENDIF	;	STKSIZE
LabelW	_tos, public			; Mark the top of stack
stack	ends

	end	_startup
