/*
//		KERNEL.C		TDREM Debug Kernel Run-Time Customization Support
//
//		This module contains an interrupt handler to permit user control
//		over the operation of the debug kernel.  Because it is system dependent,
//		it must be customized to the requirements of the application.
//
//		This module must be linked in with the application to operate.
*/

#if	defined(TDREM)						/* Only build when TDREM is defined */
#include	<dos.h>							/* Far pointer manipulation macros */
#include	"kernel.h"						/* TDREM kernel interface definition */


/*
//		Function:	KERNELsetvect
//
//		This function installs a vector into the interrupt vector table.
*/

void		KERNELsetvect(
				unsigned		wVec,
				void far		*pVect
			)
{
	void far * far *pPtr ;

#if	defined(__TURBOC__)
	pPtr = (void far * far *) MK_FP(0x0000, wVec << 2) ;
#else
	FP_SEG(pPtr) = 0x0000 ;
	FP_OFF(pPtr) = wVec << 2 ;
#endif	/* __TURBOC__ */

	/* Poke the vector into the table */
	*pPtr = pVect ;
}


/*
//		Variables to show the inner workings of the TDREM interface.  Use the
//		Turbo Debugger watch window to examine the call counts for each function.
*/
unsigned int	wEnter ;
unsigned int	wExit ;
unsigned int	wMask ;
unsigned int	wUnmask ;
unsigned int	wNMI ;


/*
//		Function:  TDREMHandler
//
//		This function controls how the debugging kernel responds according
//		to the needs of the application program.  This code can include
//		conditional statements to handle different cases or different
//		handlers can be utilized as needed.
//
//		*** NOTE ***
//		Note that this code executes as part of the TDREM kernel.  This prevents
//		any use of breakpoints in this function.  Setting of a breakpoint in
//		this function will cause TD to hang.
//		*** NOTE ***
*/

#if	defined(__TURBOC__)
#pragma	warn	-par
void	interrupt	TDREMHandler(
			/* Registers as pushed on the stack by Turbo C/Turbo C++ */
			unsigned	bp,	unsigned	di,	unsigned	si,
			unsigned	ds,	unsigned	es,
			unsigned	dx,	unsigned	cx,	unsigned	bx,	unsigned	ax
		)
#else			/* Compile for Microsoft C */
void	interrupt	far	TDREMHandler(
			/* Registers as pushed on the stack by Microsoft C */
			unsigned	es,	unsigned	ds,
			unsigned	di,	unsigned	si,	unsigned	bp,	unsigned	sp,
			unsigned	bx,	unsigned	dx,	unsigned	cx,	unsigned	ax
		)
#endif	/* __TURBOC__ */
{
	/* AX determines which function is being called */
	switch (ax)   {
		case UI_ENTER:
			/* Actions when TDREM is entered */
			wEnter++ ;
			break ;

		case UI_EXIT:
			/* Actions when TDREM is exited */
			wExit++ ;
			break ;

		case UI_MASK:
			/* Actions before instruction trace */
			wMask++ ;
			break ;

		case UI_UNMASK:
			/* Actions after instruction trace */
			wUnmask++ ;
			break ;

		case UI_NMIRESET:
			/* Actions to reset any NMI logic */
			wNMI++ ;
			break ;

		case UI_RESET:
			/* Actions when application is reset */
			break ;

		case UI_BYE:
			/* Action to take on TD exit */
			break ;

		default:
			/* This would be an error condition */
			break ;
	}
}
#if	defined(__TURBOC__)
#pragma	warn	.par
#endif	/* __TURBOC__ */
#endif	/* TDREM */
