	page	60, 132
	name	tdrema
	title	TDREM 2.0 Startup Code

;
;	Startup Code for the Paradigm Turbo Debugger Remote Interface, V2.0
;	Copyright (C) 1989, 1990, 1991 Paradigm Systems.  All rights reserved.
;
;	Assemble with MASM/TASM as follows:
;		masm /mx tdrema ;
;		tasm /mx tdrema
;

IFDEF	??Version			; Check for TASM
NOWARN	RES				; Turn off reserved words warnings
ENDIF	; ??Version

	INCLUDE	startup.inc		; Macros/assembly language definitions

	subttl	Segment/class alignment and ordering section
	page
;
;	Make sure that this startup module is specified first when linking
;       TDREM.  Requires MASM 5.10 or TASM 1.0 to assemble.
;
;	Set the default segment ordering for the segments common to
;	Turbo C and Microsoft C.  Should an application need to add additional
;	segments, they should be placed here to set the position and alignment
;	characteristics of the individual segments.
;
;	The class CODE will encompass all segments generated by the
;	compiler containing executable code.  The startup code is placed
;	in a separate class so it can fit in the upper 1K of ROM for use
;	in 186/188 systems.
;
DefSeg	_text,    para,  public,  CODE,       <>
DefSeg	_data,    para,  public,  DATA,       DGROUP
DefSeg	const,    para,  public,  CONST,      DGROUP
DefSeg	_bss,     para,  public,  BSS,        DGROUP
DefSeg	_bssend,  para,  public,  BSSEND,     DGROUP
DefSeg	_stack,   para,  stack,   STACK,      DGROUP
DefSeg	romdata,  para,  public,  ROMDATA,    <>
DefSeg	erdata,   para,  public,  ENDROMDATA, <>

;
;	Declare the external references that depend on the memory model.
;
	extrn	_main : near		; This is small memory model
	assume	cs:_text

	subttl	PC ROM BIOS extension initialization code
	page
;
;	This is the ROM BIOS extension.  It simply hooks interrupt vector
;	19h so the BIOS will boot into the debugging kernel when the BIOS
;	initialization is complete.
;
_text	segment
	db	055h			; ROM BIOS signature
	db	0aah
	db	16			; 8 KB BIOS extension

;
;	Install the TDREM entry point into vector 19h and return
;

BegProc	__BIOSinit, far
	push	ds			; Save the important registers
	push	si

	xor	ax, ax			; Point to the interrupt vector
	mov	ds, ax
	mov	si, 19h * 4

	mov	wptr [si], offset __startup
	mov	wptr [si+2], seg __startup

	pop	si			; Restore the registers and exit
	pop	ds
	ret
EndProc	__BIOSinit

_text	ends


	subttl	TDREM startup code
	page
public	__acrtused
	__acrtused = 9876h		; Needed to satisfy our MSC users

_text	segment
BegProc	__startup, far
	cli				; Disable interrupts
	cld				; Make sure the direction is forward

;	*** NOTE ***
;	If the target system has dynamic memory, it must be initialized
;	here, before the startup code sets up the stack and performs
;	the compiler memory initialization.  We include a delay here
;	in case hardware prevents memory accesses till everything is
;	settled down.
;	*** NOTE ***
	mov	cx, 0			; Initial 64K count
l1:	loop	l1			; Just waste time till memory can
l2:	loop	l2			; can be accessed

;
;	Set up the stack
;
	mov	ax, dgroup		; Get the frame number for the group
	mov	ss, ax			; Load in the stack segment register
	mov	sp, offset dgroup:tos	; And the offset within the group
	assume	ss:dgroup

;
;	Prepare the segment registers for initialization.  The initialized
;	data is assumed to have been duplicated to the class ROMDATA.
;
	mov	ax, dgroup		; Get the target segment for DGROUP
	mov	es, ax
	mov	ax, romdata		; Get the segment for the copy of DGROUP
	mov	ds, ax

;
;	Copy DGROUP from its position in ROM (just after the code class)
;	to the target address in RAM
;
	mov	si, offset dgroup:idata	; Get the starting offset of the DATA segment
	mov	di, si			; Same offset, different segment
	mov	cx, offset dgroup:bdata	; Get the ending offset (start of BSS)
	sub	cx, di			; Subtract the two for a length
	shr	cx, 1			; Convert to the number of words
	rep	movsw			; Copy initialized data from ROM to RAM
	push	es			; Get the value of DGROUP
	pop	ds			; Now it is in DS
	assume	ds:dgroup, es:nothing

;
;	Zero the BSS area
;
	xor	ax, ax			; Use a zero fill pattern
	mov	di, offset dgroup:bdata	; Get the start of the BSS class
	mov	cx, offset dgroup:edata	; Get the end address
	sub	cx, di			; Subtract the two for a length
	shr	cx, 1			; Convert to the number of words
	rep	stosw			; Zero out the BSS

;
;	Call the C language entry point of TDREM
;
	sti
	call	_main

;
;	Restart the process should main() return.  Normal practice would
;	be to have main() or some other function implement an endless while
;	loop.  This action is system dependent.
;
	jmp	__startup		; Just restart if we return

EndProc	__startup

_text	ends

	subttl	Data declarations
	page

romdata	segment
public	ridata
ridata	label	byte			; Mark the start of the ROM data area
romdata	ends

erdata	segment
	db	16 dup (?)		; Force the next segment to a new paragraph
erdata	ends

_data	segment
public	idata
idata	label	byte			; Mark the start of the _DATA segment
_data	ends

_bss	segment
public	bdata
bdata	label	byte			; Mark the start of the BSS class
_bss	ends

_bssend segment
public	edata
edata	label	byte			; Mark the end of the BSS class
_bssend ends

_stack	segment
public	_theEnd				; Used to find the end of the stack
	dw	128 dup (?)		; Declare the stack and the heap size
tos	label	word			; Define the top of stack (the initial SP value)
_theEnd	label	byte
_stack	ends

	end	__startup		; This is the entry point
