/*
//		I80186.H		Definitions for the Intel 80186/188 microprocessors
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the Intel 80186/188 (the CMOS variants),
//		including the 80C186EA and the 80C186XL device.
//		These definitions are for I/O mapped peripheral registers.
//
//		This module can be customized by defining the following symbols before
//		including this file.                                            ======
//
//			IOB186	 Peripheral register I/O base address
//	
//		Failure to define this constant results in an error.
*/

#if	!defined(_I80186)
#define	_I80186

/*
//		Check that the required symbols are defined
*/
#if	!defined(IOB186)
#error	Must define the 80186/188 I/O base address
#endif

/*
//		Define the system peripheral port addresses
*/
#define	RELOC		IOB186 + 0xfe		/* Relocation register */
#define	MPCS		IOB186 + 0xa8		/* Chip selection control registers */
#define	MMCS		IOB186 + 0xa6
#define	PACS		IOB186 + 0xa4
#define	LMCS		IOB186 + 0xa2
#define	UMCS		IOB186 + 0xa0

#define	MDRAM		IOB186 + 0xe0		/* 80C186 extensions */
#define	CDRAM		IOB186 + 0xe2
#define	EDRAM		IOB186 + 0xe4

#define	T0CT		IOB186 + 0x50		/* Timer registers */
#define	T0CTA		IOB186 + 0x52
#define	T0CTB		IOB186 + 0x54
#define	T0MD		IOB186 + 0x56
#define	T1CT		IOB186 + 0x58
#define	T1CTA		IOB186 + 0x5a
#define	T1CTB		IOB186 + 0x5c
#define	T1MD		IOB186 + 0x5e
#define	T2CT		IOB186 + 0x60
#define	T2CTA		IOB186 + 0x62
#define	T2MD		IOB186 + 0x66

/* Master mode interrupt controller port definitions */
#define	EOI		IOB186 + 0x22		/* End of interrupt register */
#define	POLL		IOB186 + 0x24		/* Poll register */
#define	PSTAT		IOB186 + 0x26		/* Poll status register */
#define	MASK		IOB186 + 0x28		/* Interrupt mask register */
#define	PRIMASK	IOB186 + 0x2a		/* Priority mask register */
#define	INSER		IOB186 + 0x2c		/* In-service register */
#define	INTREQ	IOB186 + 0x2e		/* Interrupt request register */
#define	ISTAT		IOB186 + 0x30		/* Interrupt status register */
#define	TMRCTL	IOB186 + 0x32		/* Timer control register */
#define	DMA0CTL	IOB186 + 0x34		/* DMA 0 control register */
#define	DMA1CTL	IOB186 + 0x36		/* DMA 1 control register */
#define	INT0CTL	IOB186 + 0x38		/* INT0 control register */
#define	INT1CTL	IOB186 + 0x3a		/* INT1 control register */
#define	INT2CTL	IOB186 + 0x3c		/* INT2 control register */
#define	INT3CTL	IOB186 + 0x3e		/* INT3 control register */

/* Interrupt vector numbers */
#define	INT0NUM		12					/* INT0 interrupt vector */
#define	INT1NUM		13					/* INT1 interrupt vector */
#define	INT2NUM		14					/* INT2 interrupt vector */
#define	INT3NUM		15					/* INT3 interrupt vector */

#endif	/* _I80186 */
