/*
//		Processor definition file for the Intel 80186EB microprocessor
//		Copyright (C) 1990 Paradigm Systems.  All rights reserved.
//
//		This file contains defintions for manipulating the Intel 80186EB
//		microprocessor peripheral control block.  These definitions are for
//		I/O mapped peripheral registers.
//
//		This module can be customized by defining the following symbols before
//		including this file.                                            ======
//
//			IOB186EB		Peripheral register I/O base address
//	
//		Failure to define this constant results in an error.
*/

#if	!defined(_I80186EB)
#define	_I80186EB

/*
//		Check that the required symbols are defined
*/
#if	!defined(IOB186EB)
#error	Must define the 80C186EB/C188EB I/O base address
#endif

/*
//		Peripheral Control Block register offsets
*/
#define	EOI			IOB186EB + 0x02
#define	POLL			IOB186EB + 0x04
#define	POLLSTS		IOB186EB + 0x06
#define	IMASK			IOB186EB + 0x08
#define	PRIMSK		IOB186EB + 0x0A
#define	INSERV		IOB186EB + 0x0C
#define	REQST			IOB186EB + 0x0E
#define	INTSTS		IOB186EB + 0x10
#define	TCUCON		IOB186EB + 0x12
#define	SCUCON		IOB186EB + 0x14
#define	I4CON			IOB186EB + 0x16
#define	I0CON			IOB186EB + 0x18
#define	I1CON			IOB186EB + 0x1A
#define	I2CON			IOB186EB + 0x1C
#define	I3CON			IOB186EB + 0x1E
#define	T0CNT			IOB186EB + 0x30
#define	T0CMPA		IOB186EB + 0x32
#define	T0CMPB		IOB186EB + 0x34
#define	T0CON			IOB186EB + 0x36
#define	T1CNT			IOB186EB + 0x38
#define	T1CMPA		IOB186EB + 0x3A
#define	T1CMPB		IOB186EB + 0x3C
#define	T1CON			IOB186EB + 0x3E
#define	T2CNT			IOB186EB + 0x40
#define	T2CMPA		IOB186EB + 0x42
#define	T2CON			IOB186EB + 0x46
#define	P1DIR			IOB186EB + 0x50
#define	P1PIN			IOB186EB + 0x52
#define	P1CON			IOB186EB + 0x54
#define	P1LTCH		IOB186EB + 0x56
#define	P2DIR			IOB186EB + 0x58
#define	P2PIN			IOB186EB + 0x5A
#define	P2CON			IOB186EB + 0x5C
#define	P2LTCH		IOB186EB + 0x5E
#define	B0CMP			IOB186EB + 0x60
#define	B0CNT			IOB186EB + 0x62
#define	S0CON			IOB186EB + 0x64
#define	S0STS			IOB186EB + 0x66
#define	S0RBUF		IOB186EB + 0x68
#define	S0TBUF		IOB186EB + 0x6A
#define	B1CMP			IOB186EB + 0x70
#define	B1CNT			IOB186EB + 0x72
#define	S1CON			IOB186EB + 0x74
#define	S1STS			IOB186EB + 0x76
#define	S1RBUF		IOB186EB + 0x78
#define	S1TBUF		IOB186EB + 0x7A
#define	GCS0ST		IOB186EB + 0x80
#define	GCS0SP		IOB186EB + 0x82
#define	GCS1ST		IOB186EB + 0x84
#define	GCS1SP		IOB186EB + 0x86
#define	GCS2ST		IOB186EB + 0x88
#define	GCS2SP		IOB186EB + 0x8A
#define	GCS3ST		IOB186EB + 0x8C
#define	GCS3SP		IOB186EB + 0x8E
#define	GCS4ST		IOB186EB + 0x90
#define	GCS4SP		IOB186EB + 0x92
#define	GCS5ST		IOB186EB + 0x94
#define	GCS5SP		IOB186EB + 0x96
#define	GCS6ST		IOB186EB + 0x98
#define	GCS6SP		IOB186EB + 0x9A
#define	GCS7ST		IOB186EB + 0x9C
#define	GCS7SP		IOB186EB + 0x9E
#define	LCS0ST		IOB186EB + 0xA0
#define	LCS0SP		IOB186EB + 0xA2
#define	UCS0ST		IOB186EB + 0xA4
#define	UCS0SP		IOB186EB + 0xA6
#define	RELREG		IOB186EB + 0xA8
#define	RFBASE		IOB186EB + 0xB0
#define	RFTIME		IOB186EB + 0xB2
#define	RFCON			IOB186EB + 0xB4
#define	RFADDR		IOB186EB + 0xB6
#define	PWRCON		IOB186EB + 0xB8


/* Timer control (T0CON, T1CON, T2CON) registers */
#define	TCU_CONT		0x0001			/* Enable continuous mode */
#define	TCU_ALT		0x0002			/* Alternate compare registers */
#define	TCU_EXT		0x0004			/* Enable external clocking */
#define	TCU_P			0x0008			/* Enable prescalar */
#define	TCU_RTG		0x0010			/* Enable retrigger */
#define	TCU_MC		0x0020			/* Max count occurred */
#define	TCU_RIU		0x1000			/* Register in use */
#define	TCU_INT		0x2000			/* Interrupt on terminal count */
#define	TCU_INH		0x4000			/* Permit writes enable bit */
#define	TCU_EN		0x8000			/* Enable bit */


/* Chip select channel stop (UCSSP, LSCSP, GCS0SP-GCS7SP) registers */
#define	CSU_RDY		0x0001			/* Use external ready input */
#define	CSU_MEM		0x0002			/* Select memory-mapped chip select */
#define	CSU_ISTOP	0x0004			/* Ignore stop address */
#define	CSU_CSEN		0x0008			/* Enable chip select */


/* Baud rate compare (B0CMP, B1CMP) registers */
#define	SCU_ICLK		0x8000			/* Select internal clock for BRG */

/* Serial status (S0STS, S1STS) registers */
#define	SCU_CTS		0x0002			/* CTS input value */
#define	SCU_OE		0x0004			/* Overrun error */
#define	SCU_TXE		0x0008			/* Transmitter empty */
#define	SCU_FE		0x0010			/* Framing error */
#define	SCU_TI		0x0020			/* Transmit interrupt */
#define	SCU_RI		0x0040			/* Receive interrupt */
#define	SCU_RB8PE	0x0080			/* Bit 8/parity error */
#define	SCU_DBRK0	0x0100			/* Detect break 0 */
#define	SCU_DBRK1	0x0200			/* Detect break 1 */

/* Serial control (S0CON, S1CON) registers */
#define	SCU_MODE0	0x0000			/* Select mode 0 */
#define	SCU_MODE1	0x0001			/* Select mode 1 */
#define	SCU_MODE2	0x0002			/* Select mode 2 */
#define	SCU_MODE3	0x0003			/* Select mode 3 */
#define	SCU_MODE4	0x0004			/* Select mode 4 */
#define	SCU_PEN		0x0008			/* Enable parity */
#define	SCU_EVN		0x0010			/* Select even parity */
#define	SCU_REN		0x0020			/* Enable receiver */
#define	SCU_CEN		0x0040			/* Enable CTS */
#define	SCU_TB8		0x0080			/* Enable 9th data bit */
#define	SCU_SBRK		0x0100			/* Enable sending BRK */

/* Baud rate divisors (at 16Mhz CPU) */
#define	SCU_9600		0x80cf
#define	SCU_38K		0x8033
#define	SCU_115K		0x8010


/* Interrupt control (SCUCON, TCUCON, I0CON-I4CON) registers */
#define	ICU_MSK		0x0008			/* Mask interrupt */
#define	ICU_LEVEL	0x0010			/* Enable level-sensitive interrupts */
#define	ICU_CAS		0x0020			/* Enable cascade mode */
#define	ICU_SFNM		0x0040			/* Enable special fully nested mode */

/* Interrupt request/mask/in service (REQST/IMASK/INSERV) registers */
#define	ICU_TMR		0x0001			/* TCU request/mask/in service bit */
#define	ICU_SER		0x0004			/* SCU 0 request/mask/in service bit */
#define	ICU_INT4		0x0008			/* INT4 request/mask/in service bit */
#define	ICU_INT0		0x0010			/* INT0 request/mask/in service bit */
#define	ICU_INT1		0x0020			/* INT1 request/mask/in service bit */
#define	ICU_INT2		0x0040			/* INT2 request/mask/in service bit */
#define	ICU_INT3		0x0080			/* INT3 request/mask/in service bit */

/* Interrupt poll/poll status (POLL/POLLSTS) registers */
#define	ICU_IREQ		0x8000			/* Interrupt request pending */
#define	ICU_VT0		0x0001			/* Vector type 0 */
#define	ICU_VT1		0x0002			/* Vector type 1 */
#define	ICU_VT2		0x0004			/* Vector type 2 */
#define	ICU_VT3		0x0008			/* Vector type 3 */
#define	ICU_VT4		0x0010			/* Vector type 4 */

/* End of interrupt (EOI) register */
#define	ICU_NSPEC	0x8000			/* Non-specific EOI command */

/* Interrupt status (INTSTS) register */
#define	ICU_TMR0		0x0001			/* Timer/counter 0 interrupt */
#define	ICU_TMR1		0x0002			/* Timer/counter 1 interrupt */
#define	ICU_TMR2		0x0004			/* Timer/counter 2 interrupt */
#define	ICU_SRX		0x0008			/* Serial recieve interrupt */
#define	ICU_STX		0x0010			/* Serial transmit interrupt */
#define	ICU_NMI		0x8000			/* NMI request */


/* Refresh control (REFCON) register */
#define	RCU_REN		0x8000			/* Refresh controller enable */


/* Power management control (PWRCON) register */
#define	PMU_PWRDN	0x0001			/* Select power down mode */
#define	PMU_IDLE		0x0002			/* Select idle mode */


/* Interrupt vector numbers */
#define	INT0NUM		12					/* INT0 interrupt vector */
#define	INT1NUM		13					/* INT1 interrupt vector */
#define	INT2NUM		14					/* INT2 interrupt vector */
#define	INT3NUM		15					/* INT3 interrupt vector */
#define	INT4NUM		17					/* INT4 interrupt vector */

#define	RCV0NUM		20					/* SIO 0 receive interrupt vector */
#define	XMT0NUM		21					/* SIO 0 transmit interrupt vector */

#define	T0NUM			8					/* Timer 0 */
#define	T1NUM			18					/* Timer 1 */
#define	T2NUM			19					/* Timer 2 */

#endif	/* _I80186EB */
