/*
//		Processor definition file for the Intel 80186EC microprocessor
//		Copyright (C) 1990 Paradigm Systems.  All rights reserved.
//
//		This file contains defintions for manipulating the Intel 80186EC
//		microprocessor peripheral control block.  These definitions are for
//		I/O mapped peripheral registers.
//
//		This module can be customized by defining the following symbols before
//		including this file.                                            ======
//
//			IOB186EC		Peripheral register I/O base address
//	
//		Failure to define this constant results in an error.
*/

#if	!defined(_I80186EC)
#define	_I80186EC

/*
//		Check that the required symbols are defined
*/
#if	!defined(IOB186EC)
#error	Must define the 80C186EC/C188EC I/O base address
#endif

/*
//		Peripheral Control Block register offsets (declared as a enum so
//		The debugger will be able to inspect the I/O port addresses.
*/

typedef	enum   {
		M_ICW1	=	(int) (IOB186EC + 0x00),	/* Master PIC */
		M_ICW2	=	(int) (IOB186EC + 0x02),
		M_ICW3	=	(int) (IOB186EC + 0x02),
		M_ICW4	=	(int) (IOB186EC + 0x02),
		M_OCW1	=	(int) (IOB186EC + 0x02),
		M_OCW2	=	(int) (IOB186EC + 0x00),
		M_OCW3	=	(int) (IOB186EC + 0x00),

		S_ICW1	=	(int) (IOB186EC + 0x04),	/* Slave PIC */
		S_ICW2	=	(int) (IOB186EC + 0x06),
		S_ICW3	=	(int) (IOB186EC + 0x06),
		S_ICW4	=	(int) (IOB186EC + 0x06),
		S_OCW1	=	(int) (IOB186EC + 0x06),
		S_OCW2	=	(int) (IOB186EC + 0x04),
		S_OCW3	=	(int) (IOB186EC + 0x04),

		SCUIRL	=	(int) (IOB186EC + 0x0A),	/* Miscellaneous PIC registers */
		DMAIRL	=	(int) (IOB186EC + 0x0C),
		TIMIRL	=	(int) (IOB186EC + 0x0E),

		WDTRLDH	=	(int) (IOB186EC + 0x20),	/* Watchdog monitor registers */
		WDTRLDL	=	(int) (IOB186EC + 0x22),
		WDTCNTH	=	(int) (IOB186EC + 0x24),
		WDTCNTL	=	(int) (IOB186EC + 0x26),
		WDTCLR	=	(int) (IOB186EC + 0x28),
		WDTDIS	=	(int) (IOB186EC + 0x2A),

		T0CNT		=	(int) (IOB186EC + 0x30),	/* Timer/counter registers */
		T0CMPA	=	(int) (IOB186EC + 0x32),
		T0CMPB	=	(int) (IOB186EC + 0x34),
		T0CON		=	(int) (IOB186EC + 0x36),
		T1CNT		=	(int) (IOB186EC + 0x38),
		T1CMPA	=	(int) (IOB186EC + 0x3A),
		T1CMPB	=	(int) (IOB186EC + 0x3C),
		T1CON		=	(int) (IOB186EC + 0x3E),
		T2CNT		=	(int) (IOB186EC + 0x40),
		T2CMPA	=	(int) (IOB186EC + 0x42),
		T2CON		=	(int) (IOB186EC + 0x46),

		P3DIR		=	(int) (IOB186EC + 0x48),	/* Parallel port I/O registers */
		P3PIN		=	(int) (IOB186EC + 0x4A),
		P3CON		=	(int) (IOB186EC + 0x4C),
		P3LTCH	=	(int) (IOB186EC + 0x4E),
		P1DIR		=	(int) (IOB186EC + 0x50),
		P1PIN		=	(int) (IOB186EC + 0x52),
		P1CON		=	(int) (IOB186EC + 0x54),
		P1LTCH	=	(int) (IOB186EC + 0x56),
		P2DIR		=	(int) (IOB186EC + 0x58),
		P2PIN		=	(int) (IOB186EC + 0x5A),
		P2CON		=	(int) (IOB186EC + 0x5C),
		P2LTCH	=	(int) (IOB186EC + 0x5E),

		B0CMP		=	(int) (IOB186EC + 0x60),	/* Serial I/O registers */
		B0CNT		=	(int) (IOB186EC + 0x62),
		S0CON		=	(int) (IOB186EC + 0x64),
		S0STS		=	(int) (IOB186EC + 0x66),
		S0RBUF	=	(int) (IOB186EC + 0x68),
		S0TBUF	=	(int) (IOB186EC + 0x6A),
		B1CMP		=	(int) (IOB186EC + 0x70),
		B1CNT		=	(int) (IOB186EC + 0x72),
		S1CON		=	(int) (IOB186EC + 0x74),
		S1STS		=	(int) (IOB186EC + 0x76),
		S1RBUF	=	(int) (IOB186EC + 0x78),
		S1TBUF	=	(int) (IOB186EC + 0x7A),

		GCS0ST	=	(int) (IOB186EC + 0x80),	/* Chip select registers */
		GCS0SP	=	(int) (IOB186EC + 0x82),
		GCS1ST	=	(int) (IOB186EC + 0x84),
		GCS1SP	=	(int) (IOB186EC + 0x86),
		GCS2ST	=	(int) (IOB186EC + 0x88),
		GCS2SP	=	(int) (IOB186EC + 0x8A),
		GCS3ST	=	(int) (IOB186EC + 0x8C),
		GCS3SP	=	(int) (IOB186EC + 0x8E),
		GCS4ST	=	(int) (IOB186EC + 0x90),
		GCS4SP	=	(int) (IOB186EC + 0x92),
		GCS5ST	=	(int) (IOB186EC + 0x94),
		GCS5SP	=	(int) (IOB186EC + 0x96),
		GCS6ST	=	(int) (IOB186EC + 0x98),
		GCS6SP	=	(int) (IOB186EC + 0x9A),
		GCS7ST	=	(int) (IOB186EC + 0x9C),
		GCS7SP	=	(int) (IOB186EC + 0x9E),
		LCS0ST	=	(int) (IOB186EC + 0xA0),
		LCS0SP	=	(int) (IOB186EC + 0xA2),
		UCS0ST	=	(int) (IOB186EC + 0xA4),
		UCS0SP	=	(int) (IOB186EC + 0xA6),

		RELREG	=	(int) (IOB186EC + 0xA8),	/* Relocation register */

		RFBASE	=	(int) (IOB186EC + 0xB0),	/* DRAM refresh registers */
		RFTIME	=	(int) (IOB186EC + 0xB2),
		RFCON		=	(int) (IOB186EC + 0xB4),
		RFADDR	=	(int) (IOB186EC + 0xB6),

		PWRCON	=	(int) (IOB186EC + 0xB8),	/* Power management registers */
		PWRSAV	=	(int) (IOB186EC + 0xBE),

		D0SRCL	=	(int) (IOB186EC + 0xC0),	/* DMA controller registers */
		D0SRCH	=	(int) (IOB186EC + 0xC2),
		D0DSTL	=	(int) (IOB186EC + 0xC4),
		D0DSTH	=	(int) (IOB186EC + 0xC6),
		D0TC		=	(int) (IOB186EC + 0xC8),
		D0CON		=	(int) (IOB186EC + 0xCA),
		DMAPRI	=	(int) (IOB186EC + 0xCC),
		DMAHALT	=	(int) (IOB186EC + 0xCE),
		D1SRCL	=	(int) (IOB186EC + 0xD0),
		D1SRCH	=	(int) (IOB186EC + 0xD2),
		D1DSTL	=	(int) (IOB186EC + 0xD4),
		D1DSTH	=	(int) (IOB186EC + 0xD6),
		D1TC		=	(int) (IOB186EC + 0xD8),
		D1CON		=	(int) (IOB186EC + 0xDA),
		D2SRCL	=	(int) (IOB186EC + 0xE0),
		D2SRCH	=	(int) (IOB186EC + 0xE2),
		D2DSTL	=	(int) (IOB186EC + 0xE4),
		D2DSTH	=	(int) (IOB186EC + 0xE6),
		D2TC		=	(int) (IOB186EC + 0xE8),
		D2CON		=	(int) (IOB186EC + 0xEA),
		D3SRCL	=	(int) (IOB186EC + 0xF0),
		D3SRCH	=	(int) (IOB186EC + 0xF2),
		D3DSTL	=	(int) (IOB186EC + 0xF4),
		D3DSTH	=	(int) (IOB186EC + 0xF6),
		D3TC		=	(int) (IOB186EC + 0xF8),
		D3CON		=	(int) (IOB186EC + 0xFA)
	} I186EC ;								/* 80C186EC peripheral port addresses */


#if	defined(SCUBASE)
typedef	enum   {
		BxCMP		=	(int) (IOB186EC + SCUBASE + 0x00),
		BxCNT		=	(int) (IOB186EC + SCUBASE + 0x02),
		SxCON		=	(int) (IOB186EC + SCUBASE + 0x04),
		SxSTS		=	(int) (IOB186EC + SCUBASE + 0x06),
		SxRBUF	=	(int) (IOB186EC + SCUBASE + 0x08),
		SxTBUF	=	(int) (IOB186EC + SCUBASE + 0x0A)
	} SCUPORT ;								/* Generic serial port definition */
#endif	/* defined(SCUBASE) */


/* Serial status (S0STS, S1STS) registers */
#define	SCU_CTS		0x0002			/* CTS input value */
#define	SCU_OE		0x0004			/* Overrun error */
#define	SCU_TXE		0x0008			/* Transmitter empty */
#define	SCU_FE		0x0010			/* Framing error */
#define	SCU_TI		0x0020			/* Transmit interrupt */
#define	SCU_RI		0x0040			/* Receive interrupt */
#define	SCU_RB8PE	0x0080			/* Bit 8/parity error */
#define	SCU_DBRK0	0x0100			/* Detect break 0 */
#define	SCU_DBRK1	0x0200			/* Detect break 1 */


/* Serial control (S0CON, S1CON) registers */
#define	SCU_MODE0	0x0000			/* Select mode 0 */
#define	SCU_MODE1	0x0001			/* Select mode 1 */
#define	SCU_MODE2	0x0002			/* Select mode 2 */
#define	SCU_MODE3	0x0003			/* Select mode 3 */
#define	SCU_MODE4	0x0004			/* Select mode 4 */
#define	SCU_PEN		0x0008			/* Enable parity */
#define	SCU_EVN		0x0010			/* Select even parity */
#define	SCU_REN		0x0020			/* Enable receiver */
#define	SCU_CEN		0x0040			/* Enable CTS */
#define	SCU_TB8		0x0080			/* Enable 9th data bit */
#define	SCU_SBRK		0x0100			/* Enable sending BRK */


/* Baud rate divisors (for a 16Mhz CPU, customize as necessary) */
#define	SCU_9600		0x80cf			/* 9600 baud */
#define	SCU_19K		0x8066			/* 19.2K baud */
#define	SCU_38K		0x8033			/* 38.4K baud */
#define	SCU_57K		0x8020			/* 57.6K baud */
#define	SCU_115K		0x8010			/* 115.2K baud */


/* 8259A-compatible interrupt controller definitions */
#define	EOI			0x20				/* Non-specific end of interrupt */

#endif	/* _I80186EC */
