/*
//		I8250.H		Macros and definitions for the NS8250 (and compatibles)
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the National Semiconductor 8250 serial
//		controller.  These include both I/O port definitions and internal
//		register bit field definitions.  The latter are used by combining
//		with the bit-wise OR operation as follows:
//	
//			#define	TEST		SELECT_TIMER0 || BOTH_BYTES || MODE_0
//	
//		This module can be customized by defining the following before including
//		this file.                                              ======
//
//			SC				I/O address scale factor
//			IOB8250		I/O port base address
//
//		Failure to define these constants results in an error.
*/

#if	!defined(_I8250)
#define	_I8250

/*
//		Check that the required symbols are defined
*/
#if !defined(SC) || !defined(IOB8250)
#error	Must define the constants SC and/or IOB8250 before continuing
#endif

/* Baud rate divisors (assumes PC-compatible clocking) */
#define	BAUD115K		0x01					/* 115K baud divisor */
#define	BAUD56K		0x02					/* 57.6K baud divisor */
#define	BAUD38K		0x03					/* 38.4K baud divisor */
#define	BAUD19K		0x06					/* 19.2K baud divisor */
#define	BAUD9600		0x0C					/* 9600 baud divisor */

/* I/O port addresses */
#define	RXBUF			(IOB8250+0*SC)		/* Receiver input data buffer */
#define	TXBUF			(IOB8250+0*SC)		/* Transmitter output data buffer */
#define	DIVLSB		(IOB8250+0*SC)		/* LSB of baud rate divisor */
#define	DIVMSB		(IOB8250+1*SC)		/* MSB of baud rate divisor */
#define	INTENB		(IOB8250+1*SC)		/* Interrupt enable register */
#define	INTID			(IOB8250+2*SC)		/* Interrupt identification register */
#define	LINECTRL		(IOB8250+3*SC)		/* Line control register */
#define	MDMCTRL		(IOB8250+4*SC)		/* Modem control register */
#define	MDMSTAT		(IOB8250+6*SC)		/* Modem status register */
#define	LINSTAT		(IOB8250+5*SC)		/* Line status register */

/* Bit fields in the line status register */
#define	LS_DR			0x01					/* Receiver data ready */
#define	LS_THRE		0x20					/* Transmitter holding register empty */

/* Bit fields in the line control register */
#define	LC_8BIT		0x03					/* 8-bit words */
#define	LC_BREAK		0x40					/* Enable transmit break condition */
#define	LC_DLAB		0x80					/* Divisor latch access bit */

/* Bit fields in the interrupt enable register */
#define	IE_RXINT		1						/* Receiver data available */
#define	IE_TXINT		2						/* Transmitter empty */
#define	IE_LSINT		4						/* Line status */
#define	IE_MSINT		8						/* Modem status interrupt */

/* Bit fields in modem control register */
#define	MC_INTENB	8						/* Out2 bit that actually controls interrupt enable */
#define	MC_DTR		1						/* DTR to modem */
#define	MC_RTS		2						/* RTS to modem */

/* Bit fields in modem status register */
#define	MS_CTS		0x10					/* CTS from modem */
#define	MS_DSR		0x20					/* DSR from modem */

#endif	/* _I8250 */
