/*
//		I8251.H		Macros and definitions for the Intel 8251 (and compatibles)
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the Intel 8251 serial controller.  These
//		include both I/O port definitions and internal register bit field
//		definitions.  The latter are used by combining with the bit-wise OR
//		operation as follows:
//	
//			#define	TEST		SELECT_TIMER0 || BOTH_BYTES || MODE_0
//	
//		This module can be customized by defining the following before including
//		this file.                                              ======
//
//			SC				I/O address scale factor
//			IOB8251		I/O port base address
//
//		Failure to define these constants results in an error.
*/

#if	!defined(_I8251)
#define	_I8251

/*
//		Check that the required symbols are defined
*/
#if !defined(SC) || !defined(IOB8251)
#error	Must define the constants SC and/or IOB8251 before continuing
#endif

/* I/O port addresses */
#define	SIOCMD		(IOB8251+1*SC)	/* Command I/O port address */
#define	SIOMODE		(IOB8251+1*SC)	/* Mode I/O port address */
#define	SIODATA		(IOB8251+0*SC)	/* Data I/O port address */
#define	SIOSTAT		(IOB8251+1*SC)	/* Status I/O port address */

/* Status masks */
#define	RXRDY			0x02				/* Receiver data available */
#define	TXRDY			0x01				/* Transmitter empty */

/* Command words */
#define	RXENABLE		0x04				/* Receiver enable */
#define	TXENABLE		0x01				/* Transmitter enable */

#endif	/* _I8251 */
