/*
//		I8256.H		Macros and definitions for the Intel 8256 MUART
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the Intel i8256 multifunction peripheral.
//		These include both I/O port definitions and internal register bit field
//		definitions.  The latter are used by combining with the bit-wise OR
//		operation as follows:
//	
//			#define	TEST		SELECT_TIMER0 || BOTH_BYTES || MODE_0
//
//		This module can be customized by defining the following before including
//		this file.                                              ======
//
//			SC				I/O address scale factor
//			IOB8256		I/O port base address
//
//		Failure to define these constants results in an error.
*/

#if	!defined(_I8256)
#define	_I8256

/*
//		Check that the required symbols are defined
*/
#if !defined(SC) || !defined(IOB8256)
#error	Must define the constants SC and/or IOB8256 before continuing
#endif

#define	CMD1			(IOB8256+0*SC)		/* Internal register definitions */
#define	CMD2			(IOB8256+1*SC)
#define	CMD3			(IOB8256+2*SC)
#define	MODE			(IOB8256+3*SC)
#define	P1CTL			(IOB8256+4*SC)
#define	IR				(IOB8256+5*SC)
#define	SI				(IOB8256+5*SC)
#define	RI				(IOB8256+6*SC)
#define	IA				(IOB8256+6*SC)
#define	SRB			(IOB8256+7*SC)
#define	STB			(IOB8256+7*SC)

#define	P1				(IOB8256+8*SC)
#define	P2				(IOB8256+9*SC)
#define	TIMER1		(IOB8256+10*SC)
#define	TIMER2		(IOB8256+11*SC)
#define	TIMER3		(IOB8256+12*SC)
#define	TIMER4		(IOB8256+13*SC)
#define	TIMER5		(IOB8256+14*SC)
#define	SST			(IOB8256+15*SC)
#define	MOD			(IOB8256+15*SC)

#define	RBRDY			0x40				/* Receiver data available */
#define	TBRDY			0x20				/* Transmitter empty */

#endif	/* _I8256 */
