/*
//		I8259.H		Macros and definitions for the Intel 8259A PIC
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the IBM PC 8259 interrupt controller.
//		These include both I/O port definitions and internal register bit field
//		definitions.  The latter are used by combining with the bit-wise OR
//		operation as follows:
//	
//			#define	TEST		SELECT_TIMER0 || BOTH_BYTES || MODE_0
//	
//		This module can be customized by defining the following before including
//		this file.                                              ======
//
//			SC				I/O address scale factor
//			IOB8259		I/O port base address
//
//		Failure to define these constants results in an error.
*/

#if	!defined(_I8259)
#define	_I8259

/*
//		Check that the required symbols are defined
*/
#if !defined(SC) || !defined(IOB8259)
#error	Must define the constants SC and/or IOB8259 before continuing
#endif

/*
//		Define the I/O port addresses used by the 8259 relative to the
//		base I/O address.
*/
#define	ICW1		(IOB8259+0*SC)		/* ICW1 port address */
#define	ICW2		(IOB8259+1*SC)		/* ICW2 port address */
#define	ICW3		(IOB8259+1*SC)		/* ICW3 port address */
#define	ICW4		(IOB8259+1*SC)		/* ICW4 port address */

#define	OCW1		(IOB8259+1*SC)		/* OCW1 port address */
#define	OCW2		(IOB8259+0*SC)		/* OCW2 port address */
#define	OCW3		(IOB8259+0*SC)		/* OCW3 port address */

#define	IRR		(IOB8259+0*SC)		/* Interrupt request register */
#define	ISR		(IOB8259+0*SC)		/* Interrupt in-service register */
#define	IMR		(IOB8259+1*SC)		/* Interrupt mask register */

/*
//		Define the fields of the interrupt mask/request/in-service registers
*/
#define	CHAN0		0x01					/* Bit mask for channel 0 */
#define	CHAN1		0x02					/* Bit mask for channel 1 */
#define	CHAN2		0x04					/* Bit mask for channel 2 */
#define	CHAN3		0x08					/* Bit mask for channel 3 */
#define	CHAN4		0x10					/* Bit mask for channel 4 */
#define	CHAN5		0x20					/* Bit mask for channel 5 */
#define	CHAN6		0x40					/* Bit mask for channel 6 */
#define	CHAN7		0x80					/* Bit mask for channel 7 */

/*
//		Define the bit fields for the OCW2 word
*/
#define	LEVEL_0			0x00			/* Interrupt level 0 */
#define	LEVEL_1			0x01			/* Interrupt level 1 */
#define	LEVEL_2			0x02			/* Interrupt level 2 */
#define	LEVEL_3			0x03			/* Interrupt level 3 */
#define	LEVEL_4			0x04			/* Interrupt level 4 */
#define	LEVEL_5			0x05			/* Interrupt level 5 */
#define	LEVEL_6			0x06			/* Interrupt level 6 */
#define	LEVEL_7			0x07			/* Interrupt level 7 */

/*
//		Miscellaneous commands
*/
#define	EOI				0x20			/* Non-specific EOI */
#define	SEOI				0x40			/* Specific EOI (specify level) */
#define	NO_ROTATE 		0x00			/* No rotate priority */
#define	ROTATE			0x80			/* Rotate priority */

/*
//		Define the bit fields for the OCW3 word
*/
#define	IRR_SELECT		0x0A			/* Read interrupt request register */
#define	ISR_SELECT		0x0B			/* Read in-service register */
#define	POLLING_MODE	0x0C			/* Select polling mode */
#define	READ_SPEC_MASK	0x48			/* Read special mask mode */
#define	SET_SPEC_MASK	0x68			/* Set special mask mode */

#endif	/* _I8259 */
