/*
//		I8530.H		Macros and definitions for the 8530 (and compatibles)
//		Copyright (C) 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the Zilog 8530 serial controller.  These
//		include both I/O port definitions and internal register bit field
//		definitions.  The latter are used by combining with the bit-wise OR
//		operation as follows:
//	
//			#define	TEST		SELECT_TIMER0 || BOTH_BYTES || MODE_0
//	
//		This module can be customized by defining the following before including
//		this file.                                              ======
//
//			SC				I/O address scale factor
//			IOB8530		I/O port base address
//
//		Failure to define these constants results in an error.
*/

#if	!defined(_I8530)
#define	_I8530

/*
//		Check that the required symbols are defined
*/
#if !defined(SC) || !defined(IOB8530)
#error	Must define the constants SC and/or IOB8530 before continuing
#endif

/* Macros to read/write arbitrary 8530 registers */
#define	Rd8530(x)	(HLPRoutportb(PCTRL,(x)), HLPRinportb(PCTRL))
#define	Wr8530(x,y)	(HLPRoutportb(PCTRL,(x)), HLPRoutportb(PCTRL,(y)))


/* 8530 I/O port addresses (check their definition before using) */
#define	PCTRL			(IOB8530+0*SC)		/* Read/write register 0 */
#define	PDATA			(IOB8530+1*SC)		/* Transmit/receive buffer access */

/* 8530 internal registers (for reading/writing PCTRL) */
#define	WR0			0x00
#define	WR1			0x01
#define	WR2			0x02
#define	WR3			0x03
#define	WR4			0x04
#define	WR5			0x05
#define	WR6			0x06
#define	WR7			0x07
#define	WR8			0x08
#define	WR9			0x09
#define	WR10			0x0a
#define	WR11			0x0b
#define	WR12			0x0c
#define	WR13			0x0d
#define	WR14			0x0e
#define	WR15			0x0f

#define	RR0			0x00
#define	RR1			0x01
#define	RR2			0x02
#define	RR3			0x03
#define	RR8			0x08
#define	RR10			0x0a
#define	RR12			0x0c
#define	RR13			0x0d
#define	RR15			0x0f

/* Status register bit masks */
#define	RXRDY			0x01					/* Receiver data available */
#define	TXRDY			0x04					/* Transmitter empty */

#endif  /* _I8350 */
