/*
//		V25V35.H		Definitions for the NEC V25/V35 microprocessors
//		Copyright (C) 1989, 1990 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the NEC V25/V35 microprocessors.
*/

#if	!defined(_V25V35)
#define	_V25V35

extern	void	fint(void) ;			/* Needed for interrupt-driven I/O */

/*
//		Define the interrupt vectors corresponding to internal devices
*/
#define	SIO0ERR		12					/* Serial channel 0 error interrupt */
#define	SIO0RCV		13					/* Serial channel 0 receive interrupt */
#define	SIO0XMT		14					/* Serial channel 0 receive interrupt */
#define	SIO1ERR		16					/* Serial channel 1 error interrupt */
#define	SIO1RCV		17					/* Serial channel 1 receive interrupt */
#define	SIO1XMT		18					/* Serial channel 1 receive interrupt */

/*
//		Define some masks for accessing the interrupt registers
*/
#define	INTFLAG		0x80				/* Interrupt flag */
#define	INTMASK		0x40				/* Interrupt mask bit */


/*  This structure is used to work with either serial port */
typedef	struct   {
		BYTE		rxb ;						/* Receive buffer */
		BYTE		_00[1] ;					/* Padding */
		BYTE		txb ;						/* Transmit buffer */
		BYTE		_01[2] ;					/* Padding */
		BYTE		srms ;					/* Serial receive macro service */
		BYTE		stms ;					/* Serial transmit macro service */
		BYTE		_02[1] ;					/* Padding */
		BYTE		scm ;						/* Mode register */
		BYTE		scc ;						/* Control register */
		BYTE		brg ;						/* Baud rate generator divisor */
		BYTE		sce ;						/* Serial communication error */
		BYTE		seic ;					/* Serial error interrupt request */
		BYTE		sric ;					/* Serial receive interrupt request */
		BYTE		stic ;					/* Serial transmit interrupt request */
	}	V25SIO, far *V25SIOP ;			/* Serial I/O register layout */


/* This structure maps onto the V25/V35 special function registers */
typedef	struct	{
		BYTE		p0 ;
		BYTE		pm0 ;
		BYTE		pmc0 ;
		BYTE		_01[5] ;

		BYTE		p1 ;
		BYTE		pm1 ;
		BYTE		pmc1 ;
		BYTE		_02[5] ;

		BYTE		p2 ;
		BYTE		pm2 ;
		BYTE		pmc2 ;
		BYTE		_03[37] ;

		BYTE		pt ;
		BYTE		_04[2] ;
		BYTE		pmt ;
		BYTE		_05[4] ;

		BYTE		intm ;
		BYTE		_06[3] ;

		BYTE		ems0 ;
		BYTE		ems1 ;
		BYTE		ems2 ;
		BYTE		_07[5] ;

		BYTE		exic0 ;
		BYTE		exic1 ;
		BYTE		exic2 ;
		BYTE		_08[17] ;

		BYTE		rxb0 ;
		BYTE		_09[1] ;
		BYTE		txb0 ;
		BYTE		_10[2] ;

		BYTE		srms0 ;
		BYTE		stms0 ;
		BYTE		_11[1] ;

		BYTE		scm0 ;
		BYTE		scc0 ;

		BYTE		brg0 ;
		BYTE		sce0 ;

		BYTE		seic0 ;
		BYTE		sric0 ;
		BYTE		stic0 ;
		BYTE		_12[1] ;

		BYTE		rxb1 ;
		BYTE		_13[1] ;
		BYTE		txb1 ;
		BYTE		_14[2] ;

		BYTE		srms1 ;
		BYTE		stms1 ;
		BYTE		_15[1] ;

		BYTE		scm1 ;
		BYTE		scc1 ;

		BYTE		brg1 ;
		BYTE		sce1 ;

		BYTE		seic1 ;
		BYTE		sric1 ;
		BYTE		stic1 ;
		BYTE		_16[1] ;

		WORD		tm0 ;
		WORD		md0 ;
		WORD		_17[2] ;
  		WORD		tm1 ;
		WORD		md1 ;
		WORD		_18[2] ;

		BYTE		tmc0 ;
		BYTE		tmc1 ;
		BYTE		_19[2] ;

		BYTE		tmms0 ;
		BYTE		tmms1 ;
		BYTE		tmms2 ;
		BYTE		_20[5] ;

		BYTE		tmic0 ;
		BYTE		tmic1 ;
		BYTE		tmic2 ;
		BYTE		_21[1] ;

		BYTE		dmac0 ;
		BYTE		dmam0 ;
		BYTE		dmac1 ;
		BYTE		dmam1 ;
		BYTE		_22[8] ;

		BYTE		dic0 ;
		BYTE		dic1 ;
		BYTE		_23[50] ;

 		BYTE		stbc ;
		BYTE		rfm ;
		BYTE		_24[6] ;

		WORD		wtc ;

		BYTE		flag ;
		BYTE		prc ;
		BYTE		tbic ;
		BYTE		_25[15] ;

		BYTE		ispr ;
		BYTE		_26[2] ;
		BYTE		idb ;
	}	SFR, far *SFRP ;					/* Special function register layout */

#endif	/* _V25V35 */
