/*
//		V40V50.H		Definitions for the NEC V40/V50 microprocessors
//		Copyright (C) 1989, 1993 Paradigm Systems.  All rights reserved.
//	
//		Definitions for manipulating the NEC V40/V50 microprocessors, including
//		the V40H and V50H devices.
//		This module can be customized by defining the following macros before
//		including this file.                                           ======
//
//			OPHA_BASE			OPHA initial value
//			DULA_BASE			Offset of DMAU from OPHA
//			IULA_BASE			Offset of ICU from OPHA
//			SULA_BASE			Offset of SCU from OPHA
//			TULA_BASE			Offset of TCU from OPHA
//			SC						I/O port address scale factor
*/

#if	!defined(_V40V50)
#define	_V40V50


/*
//		Define the system peripheral port addresses
*/
#define	TCKS			0xfff0			/* Timer clock selection */
#define	RFC			0xfff2			/* Refresh control */
#define	WMB			0xfff4			/* Wait state boundary */
#define	WCY1			0xfff5			/* Wait control register 1 */
#define	WCY2			0xfff6			/* Wait control register 2 */
#define	SULA			0xfff8			/* SCU I/O offset */
#define	TULA			0xfff9			/* TCU I/O offset */
#define	IULA			0xfffa			/* ICU I/O offset */
#define	DULA			0xfffb			/* DMAU I/O offset */
#define	OPHA			0xfffc			/* Peripheral high address */
#define	OPSEL			0xfffd			/* Peripheral selection */
#define	OPCN			0xfffe			/* Peripheral connection */


/*
//		Define the internal peripheral port addresses
*/
#if	defined(OPHA_BASE) && defined(IULA_BASE) && defined(SC)
#define	IIW1			(OPHA_BASE << 8) + IULA_BASE + (0 * SC)
#define	IIW2			(OPHA_BASE << 8) + IULA_BASE + (1 * SC)
#define	IIW3			(OPHA_BASE << 8) + IULA_BASE + (1 * SC)
#define	IIW4			(OPHA_BASE << 8) + IULA_BASE + (1 * SC)
#define	IMKW			(OPHA_BASE << 8) + IULA_BASE + (1 * SC)
#define	IPFW			(OPHA_BASE << 8) + IULA_BASE + (0 * SC)
#define	IMDW			(OPHA_BASE << 8) + IULA_BASE + (0 * SC)

#define	EOI			0x20				/* ICU end of interrupt command */

#endif	/* defined(OPHA_BASE) && defined(IULA_BASE) && defined(SC) */


#if	defined(OPHA_BASE) && defined(TULA_BASE) && defined(SC)
#define	TCT0			(OPHA_BASE << 8) + TULA_BASE + (0 * SC)
#define	TCT1			(OPHA_BASE << 8) + TULA_BASE + (1 * SC)
#define	TCT2			(OPHA_BASE << 8) + TULA_BASE + (2 * SC)
#define	TMD			(OPHA_BASE << 8) + TULA_BASE + (3 * SC)
#endif	/* defined(OPHA_BASE) && defined(TULA_BASE) && defined(SC) */


#if	defined(OPHA_BASE) && defined(SULA_BASE) && defined(SC)
#define	SRB			(OPHA_BASE << 8) + SULA_BASE + (0 * SC)
#define	STB			(OPHA_BASE << 8) + SULA_BASE + (0 * SC)
#define	SST			(OPHA_BASE << 8) + SULA_BASE + (1 * SC)
#define	SCM			(OPHA_BASE << 8) + SULA_BASE + (1 * SC)
#define	SMD			(OPHA_BASE << 8) + SULA_BASE + (2 * SC)
#define	SIMK			(OPHA_BASE << 8) + SULA_BASE + (3 * SC)


/* Status masks */
#define	RBRDY			0x02				/* Receiver data available */
#define	TBRDY			0x01				/* Transmitter empty */

/* Command words */
#define	RXENABLE		0x04				/* Receiver enable */
#define	TXENABLE		0x01				/* Transmitter enable */
#define	SCU_RESET	0x40				/* SCU reset command word */

/* Interrupt related definitions */
#define	SIMK_RCV		0					/* SIMK receive interrupt bit position */
#define	SIMK_XMT		1					/* SIMK xmit interrupt bit position */
#define	SCUINT		1					/* SCU interrupt input */
#endif	/* defined(OPHA_BASE) && defined(SULA_BASE) && defined(SC) */

#endif	/* _V40V50 */
