/*
//		V53.H		Definitions for the NEC V53 microprocessor
//		Copyright (C) 1990, 1991 Paradigm Systems.  All rights reserved.
//	
//		This module can be customized by defining the following macros before
//		including this file.                                           ======
//
//			OPHA_BASE			OPHA initial value
//			DULA_BASE			Offset of V53 DMAU from OPHA
//			IULA_BASE			Offset of V53 ICU from OPHA
//			SULA_BASE			Offset of V53 SCU from OPHA
//			TULA_BASE			Offset of V53 TCU from OPHA
//			SC						I/O port address scale factor
*/

#if	!defined(_V53)
#define	_V53


/*
//		Define the system peripheral port addresses
*/
#define	BSEL			0xffe0			/* Bank select register */
#define	BADR			0xffe1			/* Bank address register */
#define	XAM			0xff80			/* Address expansion mode register */

#define	TCKS			0xfff0			/* Timer clock selection */
#define	SBCR			0xfff1			/* Standby control register */
#define	RFC			0xfff2			/* Refresh control register */
#define	SULA			0xfff8			/* SCU I/O offset register */
#define	TULA			0xfff9			/* TCU I/O offset register */
#define	IULA			0xfffa			/* ICU I/O offset register */
#define	DULA			0xfffb			/* DMAU I/O offset register */
#define	OPHA			0xfffc			/* Peripheral high address register */
#define	OPSEL			0xfffd			/* Peripheral selection register */
#define	SCTL			0xfffe			/* System control register */

#define	WAC			0xffed			/* Wait memory address control register */
#define	WMB0			0xffea			/* Wait state boundary register 0 */
#define	WMB1			0xfff3			/* Wait state boundary register 1 */
#define	WCY0			0xffec			/* Wait control register 0 */
#define	WCY1			0xffeb			/* Wait control register 1 */
#define	WCY2			0xfff4			/* Wait control register 2 */
#define	WCY3			0xfff5			/* Wait control register 3 */
#define	WCY4			0xfff6			/* Wait control register 4 */

#define	BRC			0xffe9			/* Baud rate generator register */

/*
//		Define the internal peripheral port addresses
*/
#if	defined(OPHA_BASE) && defined(IULA_BASE) && defined(SC)
#define	IIW1			((OPHA_BASE << 8) + IULA_BASE + (0 * SC))
#define	IIW2			((OPHA_BASE << 8) + IULA_BASE + (1 * SC))
#define	IIW3			((OPHA_BASE << 8) + IULA_BASE + (1 * SC))
#define	IIW4			((OPHA_BASE << 8) + IULA_BASE + (1 * SC))
#define	IMKW			((OPHA_BASE << 8) + IULA_BASE + (1 * SC))
#define	IPFW			((OPHA_BASE << 8) + IULA_BASE + (0 * SC))
#define	IMDW			((OPHA_BASE << 8) + IULA_BASE + (0 * SC))

#define	EOI			0x20				/* ICU end of interrupt command */

#endif	/* defined(OPHA_BASE) && defined(IULA_BASE) && defined(SC) */


#if	defined(OPHA_BASE) && defined(TULA_BASE) && defined(SC)
#define	TCT0			((OPHA_BASE << 8) + TULA_BASE + (0 * SC))
#define	TCT1			((OPHA_BASE << 8) + TULA_BASE + (1 * SC))
#define	TCT2			((OPHA_BASE << 8) + TULA_BASE + (2 * SC))
#define	TMD			((OPHA_BASE << 8) + TULA_BASE + (3 * SC))
#endif	/* defined(OPHA_BASE) && defined(TULA_BASE) && defined(SC) */


#if	defined(OPHA_BASE) && defined(SULA_BASE) && defined(SC)
#define	SRB			((OPHA_BASE << 8) + SULA_BASE + (0 * SC))
#define	STB			((OPHA_BASE << 8) + SULA_BASE + (0 * SC))
#define	SST			((OPHA_BASE << 8) + SULA_BASE + (1 * SC))
#define	SCM			((OPHA_BASE << 8) + SULA_BASE + (1 * SC))
#define	SMD			((OPHA_BASE << 8) + SULA_BASE + (2 * SC))
#define	SIMK			((OPHA_BASE << 8) + SULA_BASE + (3 * SC))


/* Status masks */
#define	RBRDY			0x02				/* Receiver data available */
#define	TBRDY			0x01				/* Transmitter empty */

/* Command words */
#define	RXENABLE		0x04				/* Receiver enable */
#define	TXENABLE		0x01				/* Transmitter enable */
#endif	/* defined(OPHA_BASE) && defined(SULA_BASE) && defined(SC) */

#endif	/* _V53 */
