/*
//		$Revision:   1.21  $
//		$Date:   02 Jul 1993 11:46:36  $
//
//		Paradigm Systems Turbo Debugger Remote Interface (TDREM)
//		Copyright (C) 1989-1993 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This module supplies the helper functions needed to build TDREM
//		with different C compilers.  Normally there is no need to customize
//		this module but should you do so, Paradigm is not responsible for
//		providing technical support.
*/

/* This is needed to handle the differences between Borland/Microsoft */
#if	defined(__TURBOC__)
#define	ASM		asm					/* Borland C++ compile */
#else
#define	ASM		_asm					/* Microsoft C compile */
#endif

#include	<dos.h>
#include	"helpers.h"


/*
//		Function:	HLPRmovedata
//
//		This function copies data from one segment to another, anywhere in
//		the 1MB address space.  Up to 64K-1 bytes can be copied in a single
//		call to movedata().
*/

void		HLPRmovedata(
				unsigned	sseg,				/* Source segment */
				unsigned	soff,				/* Source offset */
				unsigned	dseg,				/* Destination segment */
				unsigned	doff,				/* Destination offset */
				unsigned	n					/* Number of bytes to copy */
			)
{
	ASM	{
		pushf
		push	ds
		mov	si, soff
		mov	ds, sseg
		mov	di, doff
		mov	es, dseg
		mov	cx, n
		shr	cx, 1
		cld
		rep	movsw
		adc	cx, cx
		rep	movsb
		pop	ds
		popf
	}
}


/*
//		Function:	HLPRmemset
//
//		This function fills a memory buffer with a pattern
*/

void		*HLPRmemset(
				void		*dst,				/* Memory buffer to fill */
				unsigned	pat,				/* Fill pattern */
				unsigned	len				/* # of bytes to fill */
			)
{
	ASM	{
		pushf
		cld
		push	ds
		pop	es
		mov	di, dst
		mov	cx, len
		mov	ax, pat
		rep	stosb
		popf
	}
	return	dst ;
}


/*
//		Function:	HLPRmovmem
//
//		This function copies a memory buffer to another
*/

void		HLPRmovmem(
				void		*src,				/* Source memory buffer */
				void		*dst,				/* Destination memory buffer */
				unsigned	len				/* # of bytes to copy */
			)
{
	ASM	{
		pushf
		cld
		push	ds
		pop	es
		mov	si, src
		mov	di, dst
		mov	cx, len
		rep	movsb
		popf
	}
}


/*
//		Function:	HLPRinportb
//
//		This function performs byte input for TDREM
*/

#if	defined(__TURBOC__)
#pragma	warn	-rvl
#endif	/* __TURBOC__ */
int		HLPRinportb(
				unsigned int	port		/* I/O port to read */
			)
{
	ASM	{
		mov	dx, port
		in		al, dx
		xor	ah, ah
	}
}
#if	defined(__TURBOC__)
#pragma	warn	.rvl
#endif	/* __TURBOC__ */


/*
//		Function:	HLPRinport
//
//		This function performs word input for TDREM
*/

#if	defined(__TURBOC__)
#pragma	warn	-rvl
#endif	/* __TURBOC__ */
int		HLPRinport(
				unsigned int	port		/* I/O port to read */
			)
{
	ASM	{
		mov	dx, port
		in		ax, dx
	}
}
#if	defined(__TURBOC__)
#pragma	warn	.rvl
#endif	/* __TURBOC__ */


/*
//		Function:	HLPRoutportb
//
//		This function performs byte output for TDREM
*/

void		HLPRoutportb(
				unsigned int	port,		/* I/O port to write */
				int		value				/* Port data */
			)
{
	ASM	{
		mov	dx, port
		mov	ax, value
		out	dx, al
	}
}


/*
//		Function:	HLPRoutport
//
//		This function performs word output for TDREM
*/

void		HLPRoutport(
				unsigned int	port,		/* I/O port to write */
				int		value				/* Port data */
			)
{
	ASM	{
		mov	dx, port
		mov	ax, value
		out	dx, ax
	}
}


/*
//		Function:	HLPRlrsh (Long Right Shift)
//
//		This function right shifts a 32-bit quantity the specified number
//		of places, avoiding the need for RTL helper functions.
*/

#if	defined(__TURBOC__)
#pragma	warn	-rvl
#endif	/* __TURBOC__ */
unsigned long	HLPRlrsh(
				unsigned long	data,		/* 32-bit data */
				unsigned int	count		/* Number of bit positions to shift */
			)
{
	ASM	{
		les	ax, data
		mov	dx, es
		mov	cx, count
		and	cx, 31
		jcxz	l2
	}
l1:
	ASM	{
		xor	bx, bx
		shr	dx, 1
		rcr	bx, 1
		shr	ax, 1
		or		ax, bx
		loop	l1
	}
l2:	;
}
#if	defined(__TURBOC__)
#pragma	warn	.rvl
#endif	/* __TURBOC__ */


/*
//		Function:	HLPRllshr (Long Left Shift)
//
//		This function left shifts a 32-bit quantity the specified number
//		of places, avoiding the need for RTL helper functions.
*/

#if	defined(__TURBOC__)
#pragma	warn	-rvl
#endif	/* __TURBOC__ */
unsigned long		HLPRllsh(
				unsigned long	data,		/* 32-bit data */
				unsigned int	count		/* Number of bit positions to shift */
			)
{
	ASM	{
		les	ax, data
		mov	dx, es
		mov	cx, count
		and	cx, 31
		jcxz	l2
	}
l1:
	ASM	{
		shl	dx, 1
		shl	ax, 1
		adc	dx, 0
		loop	l1
	}
l2:	;
}
#if	defined(__TURBOC__)
#pragma	warn	.rvl
#endif	/* __TURBOC__ */


/*
//		Function:	HLPRsetvect
//
//		This function installs an interrupt handler into the interrupt
//		vector table.
*/

void		HLPRsetvect(
				unsigned int	wVec,		/* Interrupt vector number */
				void	far		*ih		/* Interrupt handler to install */
			)
{
	ASM	{
		les	ax, ih
		mov	dx, es
		xor	bx, bx
		mov	es, bx
		mov	bx, wVec
		shl	bx, 1
		shl	bx, 1
		mov	es:[bx], ax
		mov	es:[bx+2], dx
	}
}


/*
//		Function:	HLPRgetvect
//
//		This function returns the contents of an entry in the interrupt
//		vector table.
*/

#if	defined(__TURBOC__)
#pragma	warn	-rvl
#endif	/* __TURBOC__ */
void	far	*HLPRgetvect(
				unsigned int	wVec		/* Interrupt vector number */
			)
{
	ASM	{
		xor	bx, bx
		mov	es, bx
		mov	bx, wVec
		shl	bx, 1
		shl	bx, 1
		mov	ax, es:[bx]
		mov	dx, es:[bx+2]
	}
}


/*
//		Function:	HLPRint86
//
//		This function calls the TDREM kernel driver with a function code in
//		the AX register.
*/

void		HLPRint86(
				unsigned 	wVec,			/* Interrupt vector number */
				unsigned		wAX			/* Contents of AX on TDREM kernel call */
			)
{
	ASM	{
		xor	bx, bx
		mov	es, bx
		mov	bx, wVec
		shl	bx, 1
		shl	bx, 1
		mov	ax, wAX
		pushf
		cli
		call	dword ptr es:[bx]
	}
}


/*
//		Function:	HLPRenable
//
//		This function is called by the TDREM kernel to enable maskable
//		interrupts.  By not enabling maskable interrupts in this function,
//		the TDREM kernel can be run with interrupts disabled in polled mode.
//
//		This function is reserved for use by the TDREM kernel and should not
//		be called by any user-configurable modules.
*/

void		HLPRenable(void)
{
	enable() ;
}


/*
//		Function:	HLPRdisable
//
//		This function is called by the TDREM kernel to disable maskable
//		interrupts.
//
//		This function is reserved for use by the TDREM kernel and should not
//		be called by any user-configurable modules.
*/

void		HLPRdisable(void)
{
	disable() ;
}
