/*
//		Paradigm Systems - Turbo Debugger Interface Remote Package
//		Copyright (C) 1989-1993 Paradigm Systems.  All rights reserved.
//
//		This file defines the target system dependent portions of the
//		Paradigm Turbo Debugger Remote Interface.
*/

#if	!defined(_TARGET)
#define	_TARGET

#if	0										/* Disabled: include if TDREM UI is used */
#include	"kernel.h"						/* Application interface extensions */
#endif

#if	0										/* Define if interrupt-driven serial I/O */
#define	COMMINT		0x??				/* Change to desired interrupt vector */
#endif

#define	COMMBUFLEN	128				/* Length of a data packet - DON'T CHANGE */
#define	SEGBUF		1024				/* Space for TDREM segment descriptors */
												/* Adjust upward if load errors occur */

typedef	enum   {
		FPU_NONE,							/* No math coprocessor */
		FPU_8087,							/* Intel 8087 */
		FPU_80287,							/* Intel 80287 */
		FPU_80387,							/* Intel 80387 */
		FPU_80486,							/* Intel 80486 */
												/* *** Paradigm extensions *** */
		FPU_80187,							/* Intel 80C187 */
		FPU_72291							/* NEC uPD72291 */
	} FPU_ID ;								/* Valid math coprocessor IDs */


typedef	enum   {
		CPU_8086,							/* Intel 8086/88 */
		CPU_80186,							/* Intel 80186/188 */
		CPU_80286,							/* Intel 80286*/
		CPU_80386,							/* Intel 80386 */
		CPU_80486,							/* Intel 80486 */
												/* *** Paradigm extensions *** */
		CPU_80C186,							/* Intel 80C186/C188 */
		CPU_80C186EB,						/* Intel 80C186EB/C188EB */
		CPU_70116,							/* NEC V20/V30 */
		CPU_70216,							/* NEC V40/V50 */
		CPU_70330,							/* NEC V25/V35 */
		CPU_70136,							/* NEC V33 */
		CPU_70236,							/* NEC V53 */
		CPU_70335,							/* NEC V25+/V35+ */
		CPU_80C186XL,						/* Intel 80C186XL/C188XL */
		CPU_80C186EA,						/* Intel 80C186EA/C188EA */
		CPU_80C186EC,						/* Intel 80C186EC/C188EC */
		CPU_70423,							/* NEC V55SC */
		CPU_70433,							/* NEC V55PI */
		CPU_286LX,							/* AMD 286LX */
		CPU_70216H							/* NEC V40H/V50H */
	} CPU_ID ;								/* Valid CPU IDs */


typedef	enum   {
		DBG_PD10,							/* Paradigm DEBUG 1.0 */
		DBG_PD20,							/* Paradigm DEBUG 2.0 */
		DBG_PD30,							/* Paradigm DEBUG 3.0 */
		DBG_TD20,							/* Turbo Debugger 2.0 */
		DBG_TD201,							/* Turbo Debugger 2.01 */
		DBG_TD25,							/* Turbo Debugger 2.5 */
		DBG_TD30,							/* Turbo Debugger 3.0 */
		DBG_TD31,							/* Turbo Debugger 3.1 */
		DBG_TD32								/* Turbo Debugger 3.2 */
	} DEBUG_ID ;							/* Valid debugger IDs */



/*
//		External data references used by TDREM
*/
extern  BYTE	bGotPacket ;			/* Flag indicating a packet has arrived */
extern  BOOL	bIntDriven ;			/* TDREM supports interrupt-driven I/O */
extern  BYTE	*pCOMMpacket ;			/* Pointer into the start of packet data */
extern  void	Progtestpacket(void) ;
extern  void interrupt far COMMrcvint(void) ;


/*
//		The following data is referenced by the TDREM kernel and can be
//		customized to minimize the memory usage.
*/
extern  char	sSegD[] ;				/* Segment table storage */
extern  WORD	wSegDSize ;				/* Size in bytes of sSegD[] */


/*
//		Function prototypes for system dependent functions that can be
//		customized for the target system.
*/
void		USERnmireset(void) ;
void		USERmaskinterrupts(void) ;
void		USERunmaskinterrupts(void) ;
void		USERtdrementer(void) ;
void		USERtdremexit(void) ;
void		USERreset(void) ;
void		USERexit(void) ;
WORD		USERmemsize(void) ;
WORD		USERmembase(WORD) ;
BYTE		USERcputype(void) ;
BYTE		USERfputype(void) ;
BYTE		USERdebugger(void) ;

/*
//		Function prototypes for default interrupt/exception handlers that
//		are installed by TDREM.
*/
void		Progtraceentry(void) ;
void		Progbreakentry(void) ;
void		Prognmientry(void) ;
void		Prognovector(void) ;
void		Progoverflow(void) ;
void		Progbounds(void) ;
void		Progiotrap(void) ;
void		Progzerodivide(void) ;
void		Progescopcode(void) ;
void		Progunusedopcode(void) ;

#endif	/* _TARGET */
