/*
//		$Revision:   1.7  $
//		$Date:   06 Jul 1993 13:57:06  $
//
//		TYPEDEFS.H - General purpose definition file
//		Paradigm Systems Turbo Debugger Remote Interface (TDREM)
//		Copyright (C) 1989 Paradigm Systems.  All rights reserved.
//
//		Function		
//		========
//		This file contains the general purpose definitions common to the
//		all Paradigm applications.  By defining synonyms for the phyiscal
//		data types to be manipulated, portability between memory models
//		and machines is maximized.
//
//		Note that this file must allow all system include files and before
//		any application include files.
*/

#if	!defined(_TYPEDEFS)
#define	__TYPEDEFS

typedef	unsigned 		BIT ;			/* Use for defining bit fields */

typedef	unsigned char	BYTE ;		/* 8-bit data */
typedef	unsigned int	WORD ;		/* 16-bit data */
typedef	unsigned long	DWORD ;		/* 32-bit data */

typedef	void far *		PTR ;			/* Pointer to any data type */
typedef	BYTE far *		BYTE_PTR ;	/* Pointer to 8-bit data */
typedef	WORD far *		WORD_PTR ;	/* Pointer to 16-bit data */
typedef	DWORD far *		DWORD_PTR ;	/* Pointer to 32-bit data */

/*
//		Define the boolean data type
*/
typedef	enum { FALSE = 0, TRUE } BOOL ;


/*
//		Define a union for manipulating 32-bit integers
*/
typedef	union   {
		struct   {
			WORD		lw ;					/* Lower 16-bits access mode */
			WORD		uw ;					/* Upper 16-bits access mode */
		} s ;									/* Union definition */
		DWORD		ul ;						/* 32-bit access mode */
	} ULS ;									/* 32-bit address manipulation */


/*
//		Define some common macros that show up everywhere.  These are
//
//			lowbyte(x)		Extract the low order byte of WORD x
//			highbyte(x)		Extract the high order byte of WORD x
//			dim(x)			Computes the dimension of an array
*/

#if	!defined(min)
#define max(a,b)	(((a) > (b)) ? (a) : (b))
#define min(a,b)	(((a) < (b)) ? (a) : (b))
#endif

#define	lowbyte(word)		((word) & 0xff)
#define	highbyte(word)		lowbyte((word) >> 8)
#define	dim(x)				(sizeof(x)/sizeof(x[0]))

#if	!defined(MK_FP)
#define	MK_FP(x,y)	((void far *)(((unsigned long)(x)<<16)|(unsigned)(y)))
#endif	/* MK_FP */

/*
//		These macros can be used to access interrupt vector entries directly.
*/
#define	setvect(inum, addr) *(void interrupt (far * far *)())((inum) * 4) = (addr)
#define	getvect(inum) *(void interrupt (far * far *)())((inum) * 4)

#endif	/* _TYPEDEFS */
